from ase import Atoms
from ase.build import bulk
import numpy as np

def conventional_to_primitive_bcc(conv_atoms):
    """
    Convert a conventional BCC Atoms object to its primitive cell representation.
    
    :param conv_atoms: ASE Atoms object with conventional BCC cell
    :return: ASE Atoms object with primitive BCC cell
    """
    conv_cell = conv_atoms.get_cell()
    a1, a2, a3 = conv_cell
    
    # Calculate primitive cell vectors
    p1 = 0.5 * (a1 - a2 + a3)
    p2 = 0.5 * (a1 + a2 - a3)
    p3 = 0.5 * (-a1 + a2 + a3)
    prim_cell = np.array([p1, p2, p3])
    
    # Create new Atoms object with primitive cell
    prim_atoms = Atoms(
        symbols=conv_atoms.symbols[0],  # Take only one atom for primitive cell
        cell=prim_cell,
        pbc=True
    )
    prim_atoms.set_scaled_positions([(0, 0, 0)])  # Set atom at origin of primitive cell
    
    return prim_atoms

def primitive_to_conventional_bcc(prim_atoms):
    """
    Convert a primitive BCC Atoms object to its conventional cell representation.
    
    :param prim_atoms: ASE Atoms object with primitive BCC cell
    :return: ASE Atoms object with conventional BCC cell
    """
    prim_cell = prim_atoms.get_cell()
    p1, p2, p3 = prim_cell
    
    # Calculate conventional cell vectors
    a1 = p1 + p2 - p3
    a2 = -p1 + p2 + p3
    a3 = p1 - p2 + p3
    conv_cell = np.array([a1, a2, a3])
    
    # Create new Atoms object with conventional cell
    conv_atoms = Atoms(
        symbols=prim_atoms.symbols * 2,  # BCC conventional cell has 2 atoms
        cell=conv_cell,
        pbc=True
    )
    
    # Set atomic positions for conventional cell
    conv_atoms.set_scaled_positions([
        (0, 0, 0),    # Corner
        (0.5, 0.5, 0.5)  # Body center
    ])
    
    return conv_atoms

# Example usage
# Create a conventional BCC cell (e.g., for iron)
fe_conv = bulk('Fe', 'bcc', a=2.87, cubic=True)
print("Conventional BCC:")
print(fe_conv.cell)
print(f"Number of atoms: {len(fe_conv)}")

# Convert to primitive
fe_prim = conventional_to_primitive_bcc(fe_conv)
print("\nPrimitive BCC:")
print(fe_prim.cell)
print(f"Number of atoms: {len(fe_prim)}")

# Convert back to conventional
fe_conv_reconstructed = primitive_to_conventional_bcc(fe_prim)
print("\nReconstructed Conventional BCC:")
print(fe_conv_reconstructed.cell)
print(f"Number of atoms: {len(fe_conv_reconstructed)}")

# Check if reconstruction matches original
print("\nCheck if reconstruction matches original:")
print(np.allclose(fe_conv.cell, fe_conv_reconstructed.cell))
print(f"Original atoms: {fe_conv.get_chemical_symbols()}")
print(f"Reconstructed atoms: {fe_conv_reconstructed.get_chemical_symbols()}")

# Additional check: Verify that the primitive cell volume is half the conventional cell volume
conv_volume = fe_conv.get_volume()
prim_volume = fe_prim.get_volume()
print(f"\nConventional cell volume: {conv_volume:.4f}")
print(f"Primitive cell volume: {prim_volume:.4f}")
print(f"Ratio (should be close to 2): {conv_volume / prim_volume:.4f}")
