from ase import Atoms
from ase.build import bulk
import numpy as np

def conventional_to_primitive_fcc(conv_atoms):
    """
    Convert a conventional FCC Atoms object to its primitive cell representation.
    
    :param conv_atoms: ASE Atoms object with conventional FCC cell
    :return: ASE Atoms object with primitive FCC cell
    """
    conv_cell = conv_atoms.get_cell()
    a1, a2, a3 = conv_cell
    
    # Calculate primitive cell vectors
    p1 = 0.5 * (a2 + a3)
    p2 = 0.5 * (a1 + a3)
    p3 = 0.5 * (a1 + a2)
    prim_cell = np.array([p1, p2, p3])
    
    # Create new Atoms object with primitive cell
    prim_atoms = Atoms(
        symbols=conv_atoms.symbols[0],  # Take only one atom for primitive cell
        cell=prim_cell,
        pbc=True
    )
    prim_atoms.set_scaled_positions([(0, 0, 0)])  # Set atom at origin of primitive cell
    
    return prim_atoms

def primitive_to_conventional_fcc(prim_atoms):
    """
    Convert a primitive FCC Atoms object to its conventional cell representation.
    
    :param prim_atoms: ASE Atoms object with primitive FCC cell
    :return: ASE Atoms object with conventional FCC cell
    """
    prim_cell = prim_atoms.get_cell()
    p1, p2, p3 = prim_cell
    
    # Calculate conventional cell vectors
    a1 = -p1 + p2 + p3
    a2 = p1 - p2 + p3
    a3 = p1 + p2 - p3
    conv_cell = np.array([a1, a2, a3])
    
    # Create new Atoms object with conventional cell
    conv_atoms = Atoms(
        symbols=prim_atoms.symbols * 4,  # FCC conventional cell has 4 atoms
        cell=conv_cell,
        pbc=True
    )
    
    # Set atomic positions for conventional cell
    conv_atoms.set_scaled_positions([
        (0, 0, 0),      # Corner
        (0.5, 0.5, 0),  # Face center
        (0.5, 0, 0.5),  # Face center
        (0, 0.5, 0.5)   # Face center
    ])
    
    return conv_atoms

# Example usage
# Create a conventional FCC cell (e.g., for copper)
cu_conv = bulk('Cu', 'fcc', a=3.6, cubic=True)
print("Conventional FCC:")
print(cu_conv.cell)
print(f"Number of atoms: {len(cu_conv)}")

# Convert to primitive
cu_prim = conventional_to_primitive_fcc(cu_conv)
print("\nPrimitive FCC:")
print(cu_prim.cell)
print(f"Number of atoms: {len(cu_prim)}")

# Convert back to conventional
cu_conv_reconstructed = primitive_to_conventional_fcc(cu_prim)
print("\nReconstructed Conventional FCC:")
print(cu_conv_reconstructed.cell)
print(f"Number of atoms: {len(cu_conv_reconstructed)}")

# Check if reconstruction matches original
print("\nCheck if reconstruction matches original:")
print(np.allclose(cu_conv.cell, cu_conv_reconstructed.cell))
print(f"Original atoms: {cu_conv.get_chemical_symbols()}")
print(f"Reconstructed atoms: {cu_conv_reconstructed.get_chemical_symbols()}")
