from ase import Atoms
from ase.build import bulk
import numpy as np

def extract_hcp_parameters(atoms):
    """
    Extract 'a' and 'c' parameters from an HCP Atoms object.
    
    :param atoms: ASE Atoms object with HCP structure
    :return: tuple of (a, c) parameters
    """
    cell = atoms.get_cell()
    a = np.linalg.norm(cell[0])  # Length of first basis vector
    c = np.linalg.norm(cell[2])  # Length of third basis vector
    return a, c

def create_hcp_atoms(a, c, symbol):
    """
    Create an HCP Atoms object from 'a' and 'c' parameters.
    
    :param a: 'a' lattice parameter
    :param c: 'c' lattice parameter
    :param symbol: chemical symbol of the element
    :return: ASE Atoms object with HCP structure
    """
    # Create the cell vectors
    cell = np.array([
        [a, 0, 0],
        [-a/2, a*np.sqrt(3)/2, 0],
        [0, 0, c]
    ])
    
    # Create the Atoms object
    atoms = Atoms(symbols=[symbol]*2,  # HCP has 2 atoms in the unit cell
                  cell=cell,
                  pbc=True)
    
    # Set the atomic positions
    atoms.set_scaled_positions([
        (1/3, 2/3, 1/4),
        (2/3, 1/3, 3/4)
    ])
    
    return atoms

# Example usage
# Create an HCP structure (e.g., for magnesium)
mg_hcp = bulk('Mg', 'hcp', a=3.21, c=5.21)
print("Original HCP structure:")
print(mg_hcp.cell)
print(f"Number of atoms: {len(mg_hcp)}")
print(f"Atomic positions:\n{mg_hcp.get_scaled_positions()}")

# Extract parameters
a, c = extract_hcp_parameters(mg_hcp)
print(f"\nExtracted parameters: a = {a:.4f}, c = {c:.4f}")

# Recreate the structure
mg_hcp_recreated = create_hcp_atoms(a, c, 'Mg')
print("\nRecreated HCP structure:")
print(mg_hcp_recreated.cell)
print(f"Number of atoms: {len(mg_hcp_recreated)}")
print(f"Atomic positions:\n{mg_hcp_recreated.get_scaled_positions()}")

# Check if recreation matches original
print("\nCheck if recreation matches original:")
print(f"Cell match: {np.allclose(mg_hcp.cell, mg_hcp_recreated.cell)}")
print(f"Positions match: {np.allclose(mg_hcp.get_scaled_positions(), mg_hcp_recreated.get_scaled_positions())}")

# Calculate and print c/a ratio
c_a_ratio = c / a
print(f"\nc/a ratio: {c_a_ratio:.4f}")
print(f"Ideal c/a ratio: {np.sqrt(8/3):.4f}")
