from ase import Atoms
from ase.build import bulk
import numpy as np

def create_sc_structure(a, symbol):
    """
    Create a simple cubic structure.
    
    :param a: lattice constant
    :param symbol: chemical symbol of the element
    :return: ASE Atoms object with SC structure
    """
    return Atoms(symbols=[symbol],
                 positions=[[0, 0, 0]],
                 cell=[[a, 0, 0],
                       [0, a, 0],
                       [0, 0, a]],
                 pbc=True)

# Create a simple cubic structure (e.g., for polonium)
a = 3.35  # Example lattice constant for Po
po_sc = create_sc_structure(a, 'Po')

print("Simple Cubic Structure:")
print(f"Cell vectors:\n{po_sc.cell}")
print(f"Number of atoms: {len(po_sc)}")
print(f"Atomic positions:\n{po_sc.get_scaled_positions()}")

# Alternatively, using ASE's bulk function
po_sc_bulk = bulk('Po', 'sc', a=3.35)

print("\nSimple Cubic Structure using bulk function:")
print(f"Cell vectors:\n{po_sc_bulk.cell}")
print(f"Number of atoms: {len(po_sc_bulk)}")
print(f"Atomic positions:\n{po_sc_bulk.get_scaled_positions()}")

# Verify that both methods produce the same structure
print("\nCheck if both methods produce the same structure:")
print(f"Cell match: {np.allclose(po_sc.cell, po_sc_bulk.cell)}")
print(f"Positions match: {np.allclose(po_sc.positions, po_sc_bulk.positions)}")
