import numpy as np

def hexagonal_to_primitive_hcp(a, c):
    """
    Convert hexagonal axes to primitive vectors for HCP.
    
    :param a: lattice parameter a (in-plane hexagonal distance)
    :param c: lattice parameter c (out-of-plane distance)
    :return: numpy array of primitive vectors
    """
    p1 = np.array([a, 0, 0])
    p2 = np.array([-a/2, a*np.sqrt(3)/2, 0])
    p3 = np.array([0, 0, c])
    return np.array([p1, p2, p3])

def primitive_to_hexagonal_hcp(primitive_vectors):
    """
    Convert primitive vectors to hexagonal axes for HCP.
    
    :param primitive_vectors: numpy array of primitive vectors
    :return: tuple of (a, c) lattice parameters
    """
    p1, p2, p3 = primitive_vectors
    a = np.linalg.norm(p1)
    c = np.linalg.norm(p3)
    return a, c

# Example usage
a = 3.21  # Example value for Mg
c = 5.21  # Example value for Mg

primitive_vectors = hexagonal_to_primitive_hcp(a, c)
print("Primitive vectors:")
print(primitive_vectors)

reconstructed_a, reconstructed_c = primitive_to_hexagonal_hcp(primitive_vectors)
print(f"\nReconstructed hexagonal parameters:")
print(f"a = {reconstructed_a:.4f}, c = {reconstructed_c:.4f}")

print("\nCheck if reconstruction matches original:")
print(f"a: {np.isclose(a, reconstructed_a)}")
print(f"c: {np.isclose(c, reconstructed_c)}")

# Calculate and print c/a ratio
c_a_ratio = c / a
print(f"\nc/a ratio: {c_a_ratio:.4f}")
print(f"Ideal c/a ratio: {np.sqrt(8/3):.4f}")

# Calculate atomic positions in fractional coordinates
atom1 = np.array([0, 0, 0])
atom2 = np.array([1/3, 2/3, 1/2])
print("\nAtomic positions (fractional coordinates):")
print(f"Atom 1: {atom1}")
print(f"Atom 2: {atom2}")
