from ase import Atoms
from ase.data import atomic_numbers
import numpy as np

def kkr_to_ase_atoms(params):
    # Conversion factor from Bohr to Ångström
    bohr_to_ang = 0.529177
    
    # Get the lattice parameter and convert to Ångström
    alat = params.get_value('ALATBASIS') * bohr_to_ang
    
    # Get the Bravais lattice vectors
    bravais = np.array(params.get_value('BRAVAIS'))
    
    # Calculate the cell by multiplying bravais with alat
    cell = bravais * alat
    
    # Get the number of atoms
    n_atoms = params.get_value('NAEZ')
    
    # Get the atomic positions
    positions = np.array(params.get_value('<RBASIS>'))
    
    # Check if CARTESIAN is specified (defaults to False)
    cartesian = params.get_value('CARTESIAN', default=False)
    
    # Get the atomic numbers
    z_atoms = params.get_value('<ZATOM>')
    
    # Ensure z_atoms is a list
    if not isinstance(z_atoms, list):
        z_atoms = [z_atoms]
    
    # Create a dictionary to convert atomic numbers to symbols
    atomic_number_symbol = {value: key for key, value in atomic_numbers.items()}
    
    # Convert atomic numbers to symbols
    symbols = [atomic_number_symbol[int(z)] for z in z_atoms]
    
    # Create the Atoms object
    if cartesian:
        # If CARTESIAN is True, positions are in lattice constant units (Bohr)
        # Convert to Ångström and use as absolute positions
        atoms = Atoms(symbols=symbols,
                      positions=positions * alat,
                      cell=cell,
                      pbc=True)
    else:
        # If CARTESIAN is False (default), positions are in fractional coordinates
        atoms = Atoms(symbols=symbols,
                      scaled_positions=positions,
                      cell=cell,
                      pbc=True)
    
    return atoms

# Usage example
params = kkrparams()
params.read_keywords_from_inputcard('path/to/inputcard')
ase_atoms = kkr_to_ase_atoms(params)

# Print some information about the created Atoms object
print(f"Number of atoms: {len(ase_atoms)}")
print(f"Chemical formula: {ase_atoms.get_chemical_formula()}")
print(f"Cell vectors (Å):\n{ase_atoms.get_cell()}")
print(f"Atomic positions (Å):\n{ase_atoms.get_positions()}")
print(f"Atomic scaled positions:\n{ase_atoms.get_scaled_positions()}")
