import numpy as np
from scipy.interpolate import interp1d
from scipy.special import sph_harm
from tqdm import tqdm

def map_potential_to_grid_partially_vectorized(atoms, potentials, grid_density=20):
    BOHR_TO_ANGSTROM = get_aBohr2Ang()

    # Define the Cartesian grid (in Angstroms)
    cell = atoms.get_cell()
    cell_lengths = np.linalg.norm(cell, axis=1)
    grid_shape = np.ceil(cell_lengths * grid_density).astype(int)

    x = np.linspace(0, cell_lengths[0], grid_shape[0], endpoint=False)
    y = np.linspace(0, cell_lengths[1], grid_shape[1], endpoint=False)
    z = np.linspace(0, cell_lengths[2], grid_shape[2], endpoint=False)
    
    # Create meshgrid of Cartesian coordinates
    X, Y, Z = np.meshgrid(x, y, z, indexing='ij')
    grid_points = np.stack([X, Y, Z], axis=-1)

    # Initialize the potential array
    V = np.zeros(grid_shape)

    # Check if the number of potentials matches the number of atoms
    if len(atoms) != len(potentials):
        raise ValueError("Number of atoms does not match number of potential instances")

    # Prepare interpolation functions for each atom and angular momentum channel
    interp_funcs = []
    for potential in potentials:
        radial_grid_angstrom = potential.radial_grid * BOHR_TO_ANGSTROM
        interp_funcs_atom = {L: interp1d(radial_grid_angstrom, values, kind='cubic', bounds_error=False, fill_value=0) 
                      for L, values in potential.padded_data().items()}
        interp_funcs.append(interp_funcs_atom)

    # Generate all periodic image displacements
    periodic_images = np.array(np.meshgrid([-1, 0, 1], [-1, 0, 1], [-1, 0, 1])).T.reshape(-1, 3)
    
    # Main computation loop
    for idx_atom, (potential, interp_funcs_atom) in enumerate(zip(potentials, interp_funcs)):
        atom_position = atoms.get_positions()[idx_atom]
        rws_angstrom = potential.radius_full * BOHR_TO_ANGSTROM

        for image_disp in periodic_images:
            # Calculate the position of the periodic image
            image_position = atom_position + np.dot(cell, image_disp)
            
            # Calculate vectors from grid points to atom image
            r_vectors = grid_points - image_position
            
            # Calculate distances
            r = np.linalg.norm(r_vectors, axis=-1)
            
            # Mask for points within Wigner-Seitz radius
            mask = r <= rws_angstrom
            
            if np.any(mask):
                r_masked = r[mask]
                r_vectors_masked = r_vectors[mask]

                # Calculate spherical coordinates for points within mask
                theta = np.arccos(np.clip(r_vectors_masked[..., 2] / r_masked, -1, 1))
                phi = np.arctan2(r_vectors_masked[..., 1], r_vectors_masked[..., 0])

                # Sum contributions from all angular momentum channels
                for L, interp_func in interp_funcs_atom.items():
                    l, m = L
                    Y_lm = sph_harm(m, l, phi, theta)
                    V[mask] += interp_func(r_masked) * Y_lm.real

    return V, (x, y, z)

# Usage:
# V, (x, y, z) = map_potential_to_grid_partially_vectorized(atoms, potentials, grid_density=20)
