import numpy as np
from scipy.interpolate import interp1d
from scipy.special import sph_harm
from ase import Atoms
from tqdm import tqdm, trange

def map_potential_to_grid(atoms, potentials, grid_density=20):
    BOHR_TO_ANGSTROM = get_aBohr2Ang()

    # Define the Cartesian grid (in Angstroms)
    cell = atoms.get_cell()
    cell_lengths = np.linalg.norm(cell, axis=1)
    grid_shape = np.ceil(cell_lengths * grid_density).astype(int)

    x = np.linspace(0, cell_lengths[0], grid_shape[0], endpoint=False)
    y = np.linspace(0, cell_lengths[1], grid_shape[1], endpoint=False)
    z = np.linspace(0, cell_lengths[2], grid_shape[2], endpoint=False)
    meshgrid = np.meshgrid(x, y, z, indexing='ij')

    # Initialize the potential array
    V = np.zeros(grid_shape)

    # Check if the number of potentials matches the number of atoms
    if len(atoms) != len(potentials):
        raise ValueError("Number of atoms does not match number of potential instances")

    # Prepare interpolation functions for each atom and angular momentum channel
    print("Preparing interpolation functions...")
    interp_funcs = []
    for potential in tqdm(potentials, desc="Atoms"):
        radial_grid_angstrom = potential.radial_grid * BOHR_TO_ANGSTROM
        interp_funcs_atom = {L: interp1d(radial_grid_angstrom, values, kind='cubic', bounds_error=False, fill_value=0) 
                      for L, values in tqdm(potential.padded_data().items(), desc="Channels", leave=False)}
        interp_funcs.append(interp_funcs_atom)

    # Get the inverse of the cell matrix for fractional coordinates
    cell_inv = np.linalg.inv(cell)

    # Loop over grid points
    print("Mapping potential to grid...")
    total_points = np.prod(grid_shape)
    with tqdm(total=total_points, desc="Grid points") as pbar:
        for i in range(grid_shape[0]):
            for j in range(grid_shape[1]):
                for k in range(grid_shape[2]):
                    grid_point = np.dot(cell.T, [x[i]/cell_lengths[0], y[j]/cell_lengths[1], z[k]/cell_lengths[2]])  # in Angstroms

                    # Loop over all atoms in the unit cell and their periodic images
                    for idx_atom, (potential, interp_funcs_atom) in enumerate(zip(potentials, interp_funcs)):
                        atom_position = atoms.get_positions()[idx_atom]  # in Angstroms
                        rws_angstrom = potential.radius_full * BOHR_TO_ANGSTROM

                        # Consider periodic images
                        for dx in [-1, 0, 1]:
                            for dy in [-1, 0, 1]:
                                for dz in [-1, 0, 1]:
                                    # Calculate the position of the periodic image
                                    image_position = atom_position + np.dot(cell.T, [dx, dy, dz])
                                    
                                    r_vector = grid_point - image_position  # in Angstroms
                                    r = np.linalg.norm(r_vector)  # in Angstroms

                                    # Check if the point is within the Wigner-Seitz radius
                                    if r <= rws_angstrom:
                                        # Calculate spherical coordinates
                                        theta = np.arccos(r_vector[2] / r) if r > 0 else 0
                                        phi = np.arctan2(r_vector[1], r_vector[0])

                                        # Sum contributions from all angular momentum channels
                                        for L, interp_func in interp_funcs_atom.items():
                                            l, m = L  # L is a tuple (l, m)
                                            V[i, j, k] += interp_func(r) * sph_harm(m, l, phi, theta).real

                    pbar.update(1)

    return V, (x, y, z)
