import numpy as np

def compare_potentials(V1, V2, rtol=1e-5, atol=1e-8):
    """
    Compare two potential arrays and return information about their differences.
    
    Args:
    V1, V2: The two potential arrays to compare
    rtol: Relative tolerance
    atol: Absolute tolerance
    
    Returns:
    A dictionary containing comparison results
    """
    # Find the indices where the arrays differ
    diff_mask = ~np.isclose(V1, V2, rtol=rtol, atol=atol)
    diff_indices = np.where(diff_mask)
    
    # Calculate the differences
    differences = V1[diff_mask] - V2[diff_mask]
    
    # Prepare the results
    results = {
        "num_different_points": np.sum(diff_mask),
        "max_difference": np.max(np.abs(differences)),
        "mean_difference": np.mean(np.abs(differences)),
        "median_difference": np.median(np.abs(differences)),
        "diff_indices": diff_indices,
        "V1_values": V1[diff_mask],
        "V2_values": V2[diff_mask],
        "differences": differences
    }
    
    return results

# Usage example:
# comparison = compare_potentials(V_p, V_p_v)
# print(f"Number of different points: {comparison['num_different_points']}")
# print(f"Maximum absolute difference: {comparison['max_difference']}")
# print(f"Mean absolute difference: {comparison['mean_difference']}")
# print(f"Median absolute difference: {comparison['median_difference']}")

# To see the first few differing points:
# for i in range(min(5, len(comparison['differences']))):
#     idx = tuple(index[i] for index in comparison['diff_indices'])
#     print(f"Index: {idx}, V_p: {comparison['V1_values'][i]}, V_p_v: {comparison['V2_values'][i]}, Difference: {comparison['differences'][i]}")
