import numpy as np
import xarray as xr
from scipy.interpolate import RegularGridInterpolator

# Example 1: Simple NumPy arrays
def numpy_coordinate_example():
    # Create data array
    data = np.random.rand(50, 60, 70)
    
    # Create coordinate arrays
    x = np.linspace(0, 1, 50)
    y = np.linspace(0, 1, 60)
    z = np.linspace(0, 1, 70)
    
    # Create meshgrid
    X, Y, Z = np.meshgrid(x, y, z, indexing='ij')
    
    return data, X, Y, Z

# Example 2: xarray DataArray
def xarray_coordinate_example():
    # Create data array with coordinates
    data = xr.DataArray(
        np.random.rand(50, 60, 70),
        dims=('x', 'y', 'z'),
        coords={
            'x': np.linspace(0, 1, 50),
            'y': np.linspace(0, 1, 60),
            'z': np.linspace(0, 1, 70)
        }
    )
    
    return data

# Example 3: SciPy's RegularGridInterpolator
def scipy_interpolator_example():
    # Create data array
    data = np.random.rand(50, 60, 70)
    
    # Create coordinate arrays
    x = np.linspace(0, 1, 50)
    y = np.linspace(0, 1, 60)
    z = np.linspace(0, 1, 70)
    
    # Create interpolator
    interpolator = RegularGridInterpolator((x, y, z), data)
    
    return interpolator

# Usage examples
numpy_data, X, Y, Z = numpy_coordinate_example()
xarray_data = xarray_coordinate_example()
scipy_interpolator = scipy_interpolator_example()

# Accessing data
print("NumPy example - value at (0.5, 0.5, 0.5):")
print(numpy_data[np.argmin(np.abs(X-0.5)), np.argmin(np.abs(Y-0.5)), np.argmin(np.abs(Z-0.5))])

print("\nxarray example - value at (0.5, 0.5, 0.5):")
print(xarray_data.sel(x=0.5, y=0.5, z=0.5, method='nearest').values)

print("\nSciPy interpolator example - value at (0.5, 0.5, 0.5):")
print(scipy_interpolator([0.5, 0.5, 0.5]))
