import numpy as np
import plotly.graph_objects as go

def plotly_volume_plot(cpot, subsample_factor=1, subsample_method='uniform', max_points=1000000):
    """
    Create a Plotly volume plot from a CartesianGridPotential object with subsampling.
    
    Parameters:
    - cpot: CartesianGridPotential object
    - subsample_factor: int, factor by which to subsample the data (e.g., 2 means use every 2nd point)
    - subsample_method: str, 'uniform' or 'random'
    - max_points: int, maximum number of points to plot
    
    Returns:
    - Plotly Figure object
    """
    X = cpot.grid.meshgrids[0]
    Y = cpot.grid.meshgrids[1]
    Z = cpot.grid.meshgrids[2]
    V = cpot.grid.data
    
    original_shape = V.shape
    total_points = np.prod(original_shape)
    
    if total_points > max_points:
        subsample_factor = max(subsample_factor, int(np.ceil((total_points / max_points) ** (1/3))))
    
    if subsample_method == 'uniform':
        X_sub = X[::subsample_factor, ::subsample_factor, ::subsample_factor]
        Y_sub = Y[::subsample_factor, ::subsample_factor, ::subsample_factor]
        Z_sub = Z[::subsample_factor, ::subsample_factor, ::subsample_factor]
        V_sub = V[::subsample_factor, ::subsample_factor, ::subsample_factor]
    elif subsample_method == 'random':
        num_samples = total_points // (subsample_factor ** 3)
        indices = np.random.choice(total_points, num_samples, replace=False)
        X_sub = X.flatten()[indices]
        Y_sub = Y.flatten()[indices]
        Z_sub = Z.flatten()[indices]
        V_sub = V.flatten()[indices]
    else:
        raise ValueError("subsample_method must be either 'uniform' or 'random'")
    
    fig = go.Figure(data=go.Volume(
        x=X_sub.flatten(),
        y=Y_sub.flatten(),
        z=Z_sub.flatten(),
        value=V_sub.flatten(),
        isomin=V.min(),
        isomax=V.max(),
        opacity=0.1,
        surface_count=20,
    ))
    
    return fig

# Example usage:
# fig = plotly_volume_plot(cpot, subsample_factor=2, subsample_method='uniform')
# fig.show()
