import numpy as np
import plotly.graph_objects as go

def plotly_volume_plot(cpot, subsample_factor=1, subsample_method='uniform', max_points=50000, 
                       zero_threshold=1e-10, value_range=None, colorscale='viridis'):
    """
    Create a Plotly volume plot from a CartesianGridPotential object with subsampling options,
    value range filtering, and customizable colorscale.
    
    Parameters:
    - cpot: CartesianGridPotential object
    - subsample_factor: int, factor by which to subsample the data (e.g., 2 means use every 2nd point)
    - subsample_method: str, 'uniform' or 'random'
    - max_points: int, maximum number of points to plot (default: 50000)
    - zero_threshold: float, absolute values below this magnitude will be less visible in the volume
    - value_range: tuple of (min, max), if provided, only values within this range will be plotted
    - colorscale: str, name of the colorscale to use (default: 'viridis')
    
    Returns:
    - Plotly Figure object
    """
    X = cpot.grid.meshgrids[0]
    Y = cpot.grid.meshgrids[1]
    Z = cpot.grid.meshgrids[2]
    V = cpot.grid.data
    
    original_shape = V.shape
    total_points = np.prod(original_shape)
    
    if total_points > max_points:
        subsample_factor = max(subsample_factor, int(np.ceil((total_points / max_points) ** (1/3))))
    
    if subsample_method == 'uniform':
        X_sub = X[::subsample_factor, ::subsample_factor, ::subsample_factor]
        Y_sub = Y[::subsample_factor, ::subsample_factor, ::subsample_factor]
        Z_sub = Z[::subsample_factor, ::subsample_factor, ::subsample_factor]
        V_sub = V[::subsample_factor, ::subsample_factor, ::subsample_factor]
    elif subsample_method == 'random':
        num_samples = min(total_points // (subsample_factor ** 3), max_points)
        flat_indices = np.random.choice(total_points, num_samples, replace=False)
        indices = np.unravel_index(flat_indices, original_shape)
        X_sub = X[indices]
        Y_sub = Y[indices]
        Z_sub = Z[indices]
        V_sub = V[indices]
    else:
        raise ValueError("subsample_method must be either 'uniform' or 'random'")

    # Apply value range filter if provided
    if value_range is not None:
        value_min, value_max = value_range
        value_min = V_sub.min() if value_min is None else value_min
        value_max = V_sub.max() if value_max is None else value_max
        range_mask = (V_sub >= value_min) & (V_sub <= value_max)
        X_sub = X_sub[range_mask]
        Y_sub = Y_sub[range_mask]
        Z_sub = Z_sub[range_mask]
        V_sub = V_sub[range_mask]

    # Determine the value range for color scaling
    cmin = V_sub.min() if value_range is None or value_range[0] is None else value_range[0]
    cmax = V_sub.max() if value_range is None or value_range[1] is None else value_range[1]

    # Set isomin and isomax based on the data range
    data_range = cmax - cmin
    isomin = cmin + 0.1 * data_range
    isomax = cmax - 0.1 * data_range

    # Debugging information
    print(f"Data range: {cmin:.2e} to {cmax:.2e}")
    print(f"Number of points after filtering: {len(V_sub)}")
    print(f"isomin: {isomin:.2e}, isomax: {isomax:.2e}")

    fig = go.Figure(data=go.Volume(
        x=X_sub.flatten(),
        y=Y_sub.flatten(),
        z=Z_sub.flatten(),
        value=V_sub.flatten(),
        isomin=isomin,
        isomax=isomax,
        opacity=0.1,
        surface_count=20,
        colorscale=colorscale,
        cmin=cmin,
        cmax=cmax,
    ))
    
    # Add a scatter plot of the sampled points for the random method
    if subsample_method == 'random':
        fig.add_trace(go.Scatter3d(
            x=X_sub,
            y=Y_sub,
            z=Z_sub,
            mode='markers',
            marker=dict(
                size=2,
                color=V_sub,
                colorscale=colorscale,
                cmin=cmin,
                cmax=cmax,
                opacity=0.1
            ),
            showlegend=False
        ))
    
    fig.update_layout(
        scene=dict(
            xaxis_title='X',
            yaxis_title='Y',
            zaxis_title='Z'
        ),
        title=f'Volume Plot ({cmin:.2e} to {cmax:.2e})'
    )
    
    return fig

# Example usage:
# Plot positive values
# fig_positive = plotly_volume_plot(cpot, subsample_factor=2, value_range=(0, None), colorscale='Reds')
# fig_positive.show()

# Plot negative values
# fig_negative = plotly_volume_plot(cpot, subsample_factor=2, value_range=(None, 0), colorscale='Blues')
# fig_negative.show()
