import numpy as np
import plotly.graph_objects as go

def plotly_volume_plot(cpot, subsample_factor=1, subsample_method='uniform', max_points=50000, zero_threshold=1e-10):
    """
    Create a Plotly volume plot from a CartesianGridPotential object with subsampling options,
    visible positive and negative values, and a divergent colormap.
    
    Parameters:
    - cpot: CartesianGridPotential object
    - subsample_factor: int, factor by which to subsample the data (e.g., 2 means use every 2nd point)
    - subsample_method: str, 'uniform' or 'random'
    - max_points: int, maximum number of points to plot (default: 50000)
    - zero_threshold: float, absolute values below this magnitude will be less visible in the volume
    
    Returns:
    - Plotly Figure object
    """
    X = cpot.grid.meshgrids[0]
    Y = cpot.grid.meshgrids[1]
    Z = cpot.grid.meshgrids[2]
    V = cpot.grid.data
    
    original_shape = V.shape
    total_points = np.prod(original_shape)
    
    if total_points > max_points:
        subsample_factor = max(subsample_factor, int(np.ceil((total_points / max_points) ** (1/3))))
    
    if subsample_method == 'uniform':
        X_sub = X[::subsample_factor, ::subsample_factor, ::subsample_factor]
        Y_sub = Y[::subsample_factor, ::subsample_factor, ::subsample_factor]
        Z_sub = Z[::subsample_factor, ::subsample_factor, ::subsample_factor]
        V_sub = V[::subsample_factor, ::subsample_factor, ::subsample_factor]
    elif subsample_method == 'random':
        num_samples = min(total_points // (subsample_factor ** 3), max_points)
        flat_indices = np.random.choice(total_points, num_samples, replace=False)
        indices = np.unravel_index(flat_indices, original_shape)
        X_sub = X[indices]
        Y_sub = Y[indices]
        Z_sub = Z[indices]
        V_sub = V[indices]
    else:
        raise ValueError("subsample_method must be either 'uniform' or 'random'")

    # Determine the maximum absolute value for symmetrical color scaling
    max_abs_val = max(abs(V_sub.min()), abs(V_sub.max()))

    fig = go.Figure(data=go.Volume(
        x=X_sub.flatten(),
        y=Y_sub.flatten(),
        z=Z_sub.flatten(),
        value=V_sub.flatten(),
        isomin=zero_threshold,
        isomax=max_abs_val,
        opacity=0.1,
        surface_count=20,
        colorscale='RdBu_r',  # A divergent colormap: Blue for negative, Red for positive
        cmin=-max_abs_val,
        cmax=max_abs_val,
    ))
    
    # Add a scatter plot of the sampled points for the random method
    if subsample_method == 'random':
        non_zero_mask = np.abs(V_sub) > zero_threshold
        fig.add_trace(go.Scatter3d(
            x=X_sub[non_zero_mask],
            y=Y_sub[non_zero_mask],
            z=Z_sub[non_zero_mask],
            mode='markers',
            marker=dict(
                size=2,
                color=V_sub[non_zero_mask],
                colorscale='RdBu_r',
                cmin=-max_abs_val,
                cmax=max_abs_val,
                opacity=0.1
            ),
            showlegend=False
        ))
    
    fig.update_layout(
        scene=dict(
            xaxis_title='X',
            yaxis_title='Y',
            zaxis_title='Z'
        ),
        title='Volume Plot with Divergent Colormap'
    )
    
    return fig

# Example usage:
# fig = plotly_volume_plot(cpot, subsample_factor=2, subsample_method='uniform')
# fig = plotly_volume_plot(cpot, subsample_factor=2, subsample_method='random')
# fig.show()
