import numpy as np
import plotly.graph_objects as go

def plotly_volume_plot(cpot, subsample_factor=1, max_points=50000, zero_threshold=1e-10):
    """
    Create a Plotly volume plot from a CartesianGridPotential object with subsampling,
    zero-value transparency, and a divergent colormap.
    
    Parameters:
    - cpot: CartesianGridPotential object
    - subsample_factor: int, factor by which to subsample the data (e.g., 2 means use every 2nd point)
    - max_points: int, maximum number of points to plot (default: 50000)
    - zero_threshold: float, values below this magnitude are considered zero and made transparent
    
    Returns:
    - Plotly Figure object
    """
    X = cpot.grid.meshgrids[0]
    Y = cpot.grid.meshgrids[1]
    Z = cpot.grid.meshgrids[2]
    V = cpot.grid.data
    
    original_shape = V.shape
    total_points = np.prod(original_shape)
    
    if total_points > max_points:
        subsample_factor = max(subsample_factor, int(np.ceil((total_points / max_points) ** (1/3))))
    
    X_sub = X[::subsample_factor, ::subsample_factor, ::subsample_factor]
    Y_sub = Y[::subsample_factor, ::subsample_factor, ::subsample_factor]
    Z_sub = Z[::subsample_factor, ::subsample_factor, ::subsample_factor]
    V_sub = V[::subsample_factor, ::subsample_factor, ::subsample_factor]

    # Create a mask for non-zero values
    non_zero_mask = np.abs(V_sub) > zero_threshold
    
    # Flatten and filter the arrays
    X_plot = X_sub[non_zero_mask]
    Y_plot = Y_sub[non_zero_mask]
    Z_plot = Z_sub[non_zero_mask]
    V_plot = V_sub[non_zero_mask]

    # Determine the maximum absolute value for symmetrical color scaling
    max_abs_val = max(abs(V_plot.min()), abs(V_plot.max()))

    fig = go.Figure(data=go.Volume(
        x=X_plot,
        y=Y_plot,
        z=Z_plot,
        value=V_plot,
        isomin=-max_abs_val,
        isomax=max_abs_val,
        opacity=0.1,
        surface_count=20,
        colorscale='RdBu',  # A divergent colormap: Red for negative, Blue for positive
        cmin=-max_abs_val,
        cmax=max_abs_val,
    ))
    
    return fig

# Example usage:
# fig = plotly_volume_plot(cpot, subsample_factor=2)
# fig.show()
