import numpy as np
import plotly.graph_objects as go

def plotly_volume_plot(cpot, subsample_factor=1, subsample_method='uniform', max_points=50000, zero_threshold=1e-10):
    """
    Create a Plotly volume plot from a CartesianGridPotential object with subsampling options,
    value-based opacity, and a divergent colormap.
    
    Parameters:
    - cpot: CartesianGridPotential object
    - subsample_factor: int, factor by which to subsample the data (e.g., 2 means use every 2nd point)
    - subsample_method: str, 'uniform' or 'random'
    - max_points: int, maximum number of points to plot (default: 50000)
    - zero_threshold: float, values below this magnitude will have minimal opacity
    
    Returns:
    - Plotly Figure object
    """
    X = cpot.grid.meshgrids[0]
    Y = cpot.grid.meshgrids[1]
    Z = cpot.grid.meshgrids[2]
    V = cpot.grid.data
    
    original_shape = V.shape
    total_points = np.prod(original_shape)
    
    if total_points > max_points:
        subsample_factor = max(subsample_factor, int(np.ceil((total_points / max_points) ** (1/3))))
    
    if subsample_method == 'uniform':
        X_sub = X[::subsample_factor, ::subsample_factor, ::subsample_factor]
        Y_sub = Y[::subsample_factor, ::subsample_factor, ::subsample_factor]
        Z_sub = Z[::subsample_factor, ::subsample_factor, ::subsample_factor]
        V_sub = V[::subsample_factor, ::subsample_factor, ::subsample_factor]
    elif subsample_method == 'random':
        num_samples = min(total_points // (subsample_factor ** 3), max_points)
        flat_indices = np.random.choice(total_points, num_samples, replace=False)
        indices = np.unravel_index(flat_indices, original_shape)
        X_sub = X[indices]
        Y_sub = Y[indices]
        Z_sub = Z[indices]
        V_sub = V[indices]
    else:
        raise ValueError("subsample_method must be either 'uniform' or 'random'")

    # Determine the maximum absolute value for symmetrical color scaling
    max_abs_val = max(abs(V_sub.min()), abs(V_sub.max()))

    # Create an opacity array based on the absolute values
    opacities = np.clip(np.abs(V_sub) / max_abs_val, zero_threshold, 1)

    fig = go.Figure(data=go.Volume(
        x=X_sub.flatten(),
        y=Y_sub.flatten(),
        z=Z_sub.flatten(),
        value=V_sub.flatten(),
        isomin=-max_abs_val,
        isomax=max_abs_val,
        opacity=opacities.flatten(),  # Use the calculated opacities
        opacityscale=[[0, zero_threshold], [1, 1]],  # Map opacity values
        surface_count=20,
        colorscale='RdBu',  # A divergent colormap: Red for negative, Blue for positive
        cmin=-max_abs_val,
        cmax=max_abs_val,
    ))
    
    # Add a scatter plot of the sampled points for the random method
    if subsample_method == 'random':
        fig.add_trace(go.Scatter3d(
            x=X_sub,
            y=Y_sub,
            z=Z_sub,
            mode='markers',
            marker=dict(
                size=2,
                color=V_sub,
                colorscale='RdBu',
                cmin=-max_abs_val,
                cmax=max_abs_val,
                opacity=opacities
            ),
            showlegend=False
        ))
    
    fig.update_layout(
        scene=dict(
            xaxis_title='X',
            yaxis_title='Y',
            zaxis_title='Z'
        ),
        title='Volume Plot with Divergent Colormap'
    )
    
    return fig

# Example usage:
# fig = plotly_volume_plot(cpot, subsample_factor=2, subsample_method='uniform')
# fig = plotly_volume_plot(cpot, subsample_factor=2, subsample_method='random')
# fig.show()
