import numpy as np
import plotly.graph_objects as go

def plotly_volume_plot(data: np.ndarray, extent=None, num_surfaces=20):
    """
    Create a Plotly volume plot from a 3D numpy array.
    
    Parameters:
    - data: 3D numpy array containing the values to plot
    - extent: tuple of (xmin, xmax, ymin, ymax, zmin, zmax) defining the spatial extent of the data
              If None, will use array indices
    - num_surfaces: number of isosurfaces to render
    
    Returns:
    - Plotly Figure object
    """
    nx, ny, nz = data.shape
    
    if extent is None:
        X, Y, Z = np.meshgrid(np.arange(nx), np.arange(ny), np.arange(nz), indexing='ij')
    else:
        xmin, xmax, ymin, ymax, zmin, zmax = extent
        X, Y, Z = np.meshgrid(
            np.linspace(xmin, xmax, nx),
            np.linspace(ymin, ymax, ny),
            np.linspace(zmin, zmax, nz),
            indexing='ij'
        )
    
    fig = go.Figure(data=go.Volume(
        x=X.flatten(),
        y=Y.flatten(),
        z=Z.flatten(),
        value=data.flatten(),
        isomin=data.min(),
        isomax=data.max(),
        opacity=0.1,
        surface_count=num_surfaces,
    ))
    
    return fig

# Example usage:
# Assuming 'cpot' is your original object
# data = cpot.grid.data
# extent = (xmin, xmax, ymin, ymax, zmin, zmax)  # You'd need to get these from cpot
# fig = plotly_volume_plot(data, extent)
# fig.show()
