import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from ase import Atoms
from ipywidgets import interactive, FloatSlider

def visualize_potential_3d_interactive(atoms: Atoms, V: np.ndarray, grid: tuple, potential_range: tuple = None):
    """
    Create an interactive 3D visualization of the discretized potential within the unit cell.
    
    :param atoms: ASE Atoms object representing the unit cell
    :param V: 3D numpy array of potential values
    :param grid: Tuple of 3D meshgrid arrays (X, Y, Z) in fractional coordinates
    :param potential_range: Tuple of (vmin, vmax) for potential color scaling (optional)
    """
    cell = atoms.get_cell()
    X, Y, Z = grid
    points = np.dot(np.column_stack((X.flatten(), Y.flatten(), Z.flatten())), cell)
    
    def update_plot(azim, elev):
        fig = plt.figure(figsize=(10, 10))
        ax = fig.add_subplot(111, projection='3d')
        
        scatter = ax.scatter(points[:, 0], points[:, 1], points[:, 2], c=V.flatten(), 
                             cmap='viridis', alpha=0.5, s=5, vmin=potential_range[0], vmax=potential_range[1])
        
        plt.colorbar(scatter, label='Potential')
        
        # Plot unit cell outline
        for i in range(3):
            for j in range(3):
                if j != i:
                    for k in [0, 1]:
                        start = np.zeros(3)
                        end = np.zeros(3)
                        start[i] = k
                        end[i] = k
                        end[j] = 1
                        start_cart = np.dot(start, cell)
                        end_cart = np.dot(end, cell)
                        ax.plot([start_cart[0], end_cart[0]], 
                                [start_cart[1], end_cart[1]], 
                                [start_cart[2], end_cart[2]], 'k-')
        
        ax.set_xlabel('X (Å)')
        ax.set_ylabel('Y (Å)')
        ax.set_zlabel('Z (Å)')
        ax.set_title('Interactive 3D Visualization of Potential in Unit Cell')
        
        ax.view_init(elev=elev, azim=azim)
        ax.set_box_aspect((np.ptp(points[:, 0]), np.ptp(points[:, 1]), np.ptp(points[:, 2])))
        
        plt.tight_layout()
        plt.show()
    
    interactive_plot = interactive(update_plot, 
                                   azim=FloatSlider(min=0, max=360, step=5, value=45, description='Azimuth'),
                                   elev=FloatSlider(min=0, max=180, step=5, value=45, description='Elevation'))
    return interactive_plot

# Example usage:
# atoms = Atoms('Cu', positions=[[0, 0, 0]], cell=[[1.8, 1.8, 0], [1.8, 0, 1.8], [0, 1.8, 1.8]])
# V, grid = map_potential_to_grid(atoms, potentials)
# potential_range = (V.min(), V.max())
# interactive_plot = visualize_potential_3d_interactive(atoms, V, grid, potential_range)
# display(interactive_plot)
