import numpy as np
import plotly.graph_objects as go
from ase import Atoms

def visualize_potential_3d_plotly(atoms: Atoms, V: np.ndarray, grid: tuple, potential_range: tuple = None, max_points: int = 10000):
    """
    Create an interactive 3D visualization of the discretized potential within the unit cell using Plotly.
    
    :param atoms: ASE Atoms object representing the unit cell
    :param V: 3D numpy array of potential values
    :param grid: Tuple of 3D meshgrid arrays (X, Y, Z) in fractional coordinates
    :param potential_range: Tuple of (vmin, vmax) for potential color scaling (optional)
    :param max_points: Maximum number of points to plot (default: 10000)
    """
    cell = atoms.get_cell()
    X, Y, Z = grid
    points = np.dot(np.column_stack((X.flatten(), Y.flatten(), Z.flatten())), cell)
    values = V.flatten()
    
    # Subsample points if necessary
    if len(points) > max_points:
        indices = np.random.choice(len(points), max_points, replace=False)
        points = points[indices]
        values = values[indices]
    
    # Create scatter plot for potential values
    scatter = go.Scatter3d(
        x=points[:, 0], y=points[:, 1], z=points[:, 2],
        mode='markers',
        marker=dict(
            size=3,
            color=values,
            colorscale='Viridis',
            opacity=0.8,
            colorbar=dict(title='Potential'),
            cmin=potential_range[0] if potential_range else None,
            cmax=potential_range[1] if potential_range else None,
        )
    )
    
    # Create lines for unit cell outline
    lines_x, lines_y, lines_z = [], [], []
    for i in range(3):
        for j in range(3):
            if j != i:
                for k in [0, 1]:
                    start = np.zeros(3)
                    end = np.zeros(3)
                    start[i] = k
                    end[i] = k
                    end[j] = 1
                    start_cart = np.dot(start, cell)
                    end_cart = np.dot(end, cell)
                    lines_x.extend([start_cart[0], end_cart[0], None])
                    lines_y.extend([start_cart[1], end_cart[1], None])
                    lines_z.extend([start_cart[2], end_cart[2], None])
    
    lines = go.Scatter3d(x=lines_x, y=lines_y, z=lines_z,
                         mode='lines',
                         line=dict(color='black', width=3),
                         showlegend=False)
    
    # Combine scatter plot and lines
    data = [scatter, lines]
    
    # Set up the layout
    layout = go.Layout(
        scene=dict(
            xaxis_title='X (Å)',
            yaxis_title='Y (Å)',
            zaxis_title='Z (Å)',
            aspectmode='data'
        ),
        title='Interactive 3D Visualization of Potential in Unit Cell'
    )
    
    # Create and show the figure
    fig = go.Figure(data=data, layout=layout)
    return fig

# Example usage:
# atoms = Atoms('Cu', positions=[[0, 0, 0]], cell=[[1.8, 1.8, 0], [1.8, 0, 1.8], [0, 1.8, 1.8]])
# V, grid = map_potential_to_grid(atoms, potentials)
# potential_range = (V.min(), V.max())
# fig = visualize_potential_3d_plotly(atoms, V, grid, potential_range)
# fig.show()
