import numpy as np
import plotly.graph_objects as go
from ase import Atoms

def visualize_potential_3d_plotly_isosurface(atoms: Atoms, V: np.ndarray, grid: tuple, potential_range: tuple = None, max_points: int = 100000):
    """
    Create an interactive 3D isosurface visualization of the discretized potential within the unit cell using Plotly.
    
    :param atoms: ASE Atoms object representing the unit cell
    :param V: 3D numpy array of potential values
    :param grid: Tuple of 3D meshgrid arrays (X, Y, Z) in fractional coordinates
    :param potential_range: Tuple of (vmin, vmax) for potential color scaling (optional)
    :param max_points: Maximum number of points to plot (default: 100000)
    """
    cell = atoms.get_cell()
    X, Y, Z = grid
    
    # Convert fractional coordinates to Cartesian
    X_cart = X * cell[0, 0] + Y * cell[1, 0] + Z * cell[2, 0]
    Y_cart = X * cell[0, 1] + Y * cell[1, 1] + Z * cell[2, 1]
    Z_cart = X * cell[0, 2] + Y * cell[1, 2] + Z * cell[2, 2]
    
    # Debugging: Print detailed information about the data
    print(f"Shape of X_cart: {X_cart.shape}")
    print(f"Range of X_cart: {X_cart.min()} to {X_cart.max()}")
    print(f"Shape of Y_cart: {Y_cart.shape}")
    print(f"Range of Y_cart: {Y_cart.min()} to {Y_cart.max()}")
    print(f"Shape of Z_cart: {Z_cart.shape}")
    print(f"Range of Z_cart: {Z_cart.min()} to {Z_cart.max()}")
    print(f"Shape of V: {V.shape}")
    print(f"Range of V: {V.min()} to {V.max()}")
    print(f"Number of unique values in V: {len(np.unique(V))}")
    
    # Ensure V is not constant
    if np.allclose(V, V[0, 0, 0]):
        print("Warning: V seems to be constant. Adding small random variations.")
        V = V + np.random.normal(0, 1e-6, V.shape)
    
    # Subsample points if necessary
    if V.size > max_points:
        subsample_factor = int(np.ceil((V.size / max_points) ** (1/3)))
        X_cart = X_cart[::subsample_factor, ::subsample_factor, ::subsample_factor]
        Y_cart = Y_cart[::subsample_factor, ::subsample_factor, ::subsample_factor]
        Z_cart = Z_cart[::subsample_factor, ::subsample_factor, ::subsample_factor]
        V = V[::subsample_factor, ::subsample_factor, ::subsample_factor]
    
    # Normalize V to [0, 1] range for better visualization
    V_norm = (V - V.min()) / (V.max() - V.min())
    
    # Create isosurfaces
    isosurfaces = []
    for iso_value in np.linspace(V_norm.min(), V_norm.max(), 5):  # Create 5 isosurfaces
        isosurfaces.append(go.Isosurface(
            x=X_cart.flatten(),
            y=Y_cart.flatten(),
            z=Z_cart.flatten(),
            value=V_norm.flatten(),
            isomin=iso_value,
            isomax=iso_value,
            surface=dict(count=1),
            opacity=0.3,
            colorscale='Viridis',
            caps=dict(x_show=False, y_show=False, z_show=False)  # Hide caps
        ))
    
    # Create lines for unit cell outline
    lines_x, lines_y, lines_z = [], [], []
    for i in range(3):
        for j in range(3):
            if j != i:
                for k in [0, 1]:
                    start = np.zeros(3)
                    end = np.zeros(3)
                    start[i] = k
                    end[i] = k
                    end[j] = 1
                    start_cart = np.dot(start, cell)
                    end_cart = np.dot(end, cell)
                    lines_x.extend([start_cart[0], end_cart[0], None])
                    lines_y.extend([start_cart[1], end_cart[1], None])
                    lines_z.extend([start_cart[2], end_cart[2], None])
    
    lines = go.Scatter3d(x=lines_x, y=lines_y, z=lines_z,
                         mode='lines',
                         line=dict(color='black', width=3),
                         showlegend=False)
    
    # Combine isosurfaces and lines
    data = isosurfaces + [lines]
    
    # Set up the layout
    layout = go.Layout(
        scene=dict(
            xaxis_title='X (Å)',
            yaxis_title='Y (Å)',
            zaxis_title='Z (Å)',
            aspectmode='data'
        ),
        title='Interactive 3D Isosurface Visualization of Potential in Unit Cell'
    )
    
    # Create and show the figure
    fig = go.Figure(data=data, layout=layout)
    return fig

# Example usage:
# atoms = Atoms('Cu', positions=[[0, 0, 0]], cell=[[1.8, 1.8, 0], [1.8, 0, 1.8], [0, 1.8, 1.8]])
# V, grid = map_potential_to_grid(atoms, potentials)
# potential_range = (V.min(), V.max())
# fig = visualize_potential_3d_plotly_isosurface(atoms, V, grid, potential_range)
# fig.show()
