import numpy as np
import plotly.graph_objects as go
from ase import Atoms
from scipy import stats

def visualize_potential_3d_plotly(atoms: Atoms, V: np.ndarray, grid: tuple, potential_range: tuple = None, max_points: int = 100000):
    """
    Create an interactive 3D visualization of the discretized potential within the unit cell using Plotly.
    Adjusted for small ranges of potential values.
    
    :param atoms: ASE Atoms object representing the unit cell
    :param V: 3D numpy array of potential values
    :param grid: Tuple of 3D meshgrid arrays (X, Y, Z) in fractional coordinates
    :param potential_range: Tuple of (vmin, vmax) for potential color scaling (optional)
    :param max_points: Maximum number of points to plot (default: 100000)
    """
    cell = atoms.get_cell()
    X, Y, Z = grid
    
    # Convert fractional coordinates to Cartesian
    X_cart = X * cell[0, 0] + Y * cell[1, 0] + Z * cell[2, 0]
    Y_cart = X * cell[0, 1] + Y * cell[1, 1] + Z * cell[2, 1]
    Z_cart = X * cell[0, 2] + Y * cell[1, 2] + Z * cell[2, 2]
    
    # Subsample points if necessary
    if V.size > max_points:
        subsample_factor = int(np.ceil((V.size / max_points) ** (1/3)))
        X_cart = X_cart[::subsample_factor, ::subsample_factor, ::subsample_factor]
        Y_cart = Y_cart[::subsample_factor, ::subsample_factor, ::subsample_factor]
        Z_cart = Z_cart[::subsample_factor, ::subsample_factor, ::subsample_factor]
        V = V[::subsample_factor, ::subsample_factor, ::subsample_factor]
    
    # Normalize V using z-score normalization
    V_norm = stats.zscore(V.flatten())
    
    # Print statistics about the normalized data
    print(f"Normalized V - Mean: {V_norm.mean():.4f}, Std: {V_norm.std():.4f}")
    print(f"Normalized V - Min: {V_norm.min():.4f}, Max: {V_norm.max():.4f}")
    
    # Create volumetric plot
    volume = go.Volume(
        x=X_cart.flatten(),
        y=Y_cart.flatten(),
        z=Z_cart.flatten(),
        value=V_norm,
        isomin=-2,
        isomax=2,
        opacity=0.1,
        surface_count=21,  # Increased for more detail
        colorscale='RdBu',  # Changed to diverging colormap
        colorbar=dict(
            title='Potential (z-score)',
            tickvals=[-2, -1, 0, 1, 2],
            ticktext=['-2σ', '-1σ', 'Mean', '+1σ', '+2σ']
        )
    )
    
    # Create lines for unit cell outline
    lines_x, lines_y, lines_z = [], [], []
    for i in range(3):
        for j in range(3):
            if j != i:
                for k in [0, 1]:
                    start = np.zeros(3)
                    end = np.zeros(3)
                    start[i] = k
                    end[i] = k
                    end[j] = 1
                    start_cart = np.dot(start, cell)
                    end_cart = np.dot(end, cell)
                    lines_x.extend([start_cart[0], end_cart[0], None])
                    lines_y.extend([start_cart[1], end_cart[1], None])
                    lines_z.extend([start_cart[2], end_cart[2], None])
    
    lines = go.Scatter3d(x=lines_x, y=lines_y, z=lines_z,
                         mode='lines',
                         line=dict(color='black', width=3),
                         showlegend=False)
    
    # Combine volume plot and lines
    data = [volume, lines]
    
    # Set up the layout
    layout = go.Layout(
        scene=dict(
            xaxis_title='X (Å)',
            yaxis_title='Y (Å)',
            zaxis_title='Z (Å)',
            aspectmode='data'
        ),
        title='Interactive 3D Visualization of Potential in Unit Cell'
    )
    
    # Create and show the figure
    fig = go.Figure(data=data, layout=layout)
    return fig

# Example usage:
# atoms = Atoms('Cu', positions=[[0, 0, 0]], cell=[[1.8, 1.8, 0], [1.8, 0, 1.8], [0, 1.8, 1.8]])
# V, grid = map_potential_to_grid(atoms, potentials)
# fig = visualize_potential_3d_plotly(atoms, V, grid)
# fig.show()
