import numpy as np
import plotly.graph_objects as go
from ase import Atoms

def visualize_potential_3d_plotly(atoms: Atoms, V: np.ndarray, grid: tuple, potential_range: tuple = None):
    """
    Create an interactive 3D visualization of the discretized potential within the unit cell using Plotly.
    
    :param atoms: ASE Atoms object representing the unit cell
    :param V: 3D numpy array of potential values
    :param grid: Tuple of 3D meshgrid arrays (X, Y, Z) in fractional coordinates
    :param potential_range: Tuple of (vmin, vmax) for potential color scaling (optional)
    """
    cell = atoms.get_cell()
    X, Y, Z = grid
    points = np.dot(np.column_stack((X.flatten(), Y.flatten(), Z.flatten())), cell)
    
    # Create scatter plot for potential values
    scatter = go.Scatter3d(
        x=points[:, 0], y=points[:, 1], z=points[:, 2],
        mode='markers',
        marker=dict(
            size=3,
            color=V.flatten(),
            colorscale='Viridis',
            opacity=0.8,
            colorbar=dict(title='Potential'),
            cmin=potential_range[0] if potential_range else None,
            cmax=potential_range[1] if potential_range else None,
        )
    )
    
    # Create lines for unit cell outline
    lines = []
    for i in range(3):
        for j in range(3):
            if j != i:
                for k in [0, 1]:
                    start = np.zeros(3)
                    end = np.zeros(3)
                    start[i] = k
                    end[i] = k
                    end[j] = 1
                    start_cart = np.dot(start, cell)
                    end_cart = np.dot(end, cell)
                    lines.append(go.Scatter3d(x=[start_cart[0], end_cart[0]],
                                              y=[start_cart[1], end_cart[1]],
                                              z=[start_cart[2], end_cart[2]],
                                              mode='lines',
                                              line=dict(color='black', width=3),
                                              showlegend=False))
    
    # Combine scatter plot and lines
    data = [scatter] + lines
    
    # Set up the layout
    layout = go.Layout(
        scene=dict(
            xaxis_title='X (Å)',
            yaxis_title='Y (Å)',
            zaxis_title='Z (Å)',
            aspectmode='data'
        ),
        title='Interactive 3D Visualization of Potential in Unit Cell'
    )
    
    # Create and show the figure
    fig = go.Figure(data=data, layout=layout)
    return fig

# Example usage:
# atoms = Atoms('Cu', positions=[[0, 0, 0]], cell=[[1.8, 1.8, 0], [1.8, 0, 1.8], [0, 1.8, 1.8]])
# V, grid = map_potential_to_grid(atoms, potentials)
# potential_range = (V.min(), V.max())
# fig = visualize_potential_3d_plotly(atoms, V, grid, potential_range)
# fig.show()
