import numpy as np
from scipy.interpolate import interp1d
from scipy.special import sph_harm
from ase import Atoms

# Conversion factor
BOHR_TO_ANGSTROM = 0.529177210903  # 1 Bohr = 0.529177210903 Angstroms

class KKRPotential:
    def __init__(self, name, data, rws, radial_grid):
        self.name = name  # Chemical symbol of the atom
        self.data = data  # Dict with keys L (tuple of l, m) and values as potential values on radial grid
        self.rws = rws    # Wigner-Seitz radius for this atom (in Bohr)
        self.radial_grid = radial_grid  # Radial grid points from 0 to rws (in Bohr)

def map_potential_to_grid(atoms: Atoms, potentials: list[KKRPotential], grid_density: float = 20.0):
    # Define the Cartesian grid (in Angstroms)
    cell = atoms.get_cell()
    cell_lengths = np.linalg.norm(cell, axis=1)
    grid_shape = np.ceil(cell_lengths * grid_density).astype(int)
    
    x = np.linspace(0, 1, grid_shape[0], endpoint=False)
    y = np.linspace(0, 1, grid_shape[1], endpoint=False)
    z = np.linspace(0, 1, grid_shape[2], endpoint=False)
    grid = np.meshgrid(x, y, z, indexing='ij')
    
    # Initialize the potential array
    V = np.zeros(grid_shape)
    
    # Check if the number of potentials matches the number of atoms
    if len(atoms) != len(potentials):
        raise ValueError("Number of atoms does not match number of potential instances")

    # Prepare interpolation functions for each atom and angular momentum channel
    interp_funcs = []
    for potential in potentials:
        radial_grid_angstrom = potential.radial_grid * BOHR_TO_ANGSTROM
        atom_funcs = {L: interp1d(radial_grid_angstrom, values, kind='cubic', bounds_error=False, fill_value=0) 
                      for L, values in potential.data.items()}
        interp_funcs.append(atom_funcs)

    # Loop over grid points
    for i, j, k in np.ndindex(V.shape):
        grid_point = np.dot(cell.T, [x[i], y[j], z[k]])  # in Angstroms
        
        # Loop over all atoms in the unit cell
        for atom, (potential, atom_funcs) in enumerate(zip(potentials, interp_funcs)):
            atom_position = atoms.get_positions()[atom]  # in Angstroms
            rws_angstrom = potential.rws * BOHR_TO_ANGSTROM
            
            r_vector = grid_point - atom_position  # in Angstroms
            r = np.linalg.norm(r_vector)  # in Angstroms
            
            # Check if the point is within the Wigner-Seitz radius
            if r <= rws_angstrom:
                # Calculate spherical coordinates
                theta = np.arccos(r_vector[2] / r) if r > 0 else 0
                phi = np.arctan2(r_vector[1], r_vector[0])
                
                # Sum contributions from all angular momentum channels
                for L, interp_func in atom_funcs.items():
                    l, m = L  # L is a tuple (l, m)
                    V[i, j, k] += interp_func(r) * sph_harm(m, l, phi, theta).real
    
    return V, grid

# Example usage
atoms = Atoms('Si2', positions=[[0, 0, 0], [0.25, 0.25, 0.25]], cell=[[5.43, 0, 0], [0, 5.43, 0], [0, 0, 5.43]])

# Create example KKRPotential instances (values in Bohr)
radial_grid = np.linspace(0, 2.0, 100)  # Example radial grid in Bohr
potential1 = KKRPotential(name='Si', data={(0,0): np.random.rand(100), (1,0): np.random.rand(100)}, rws=2.0, radial_grid=radial_grid)
potential2 = KKRPotential(name='Si', data={(0,0): np.random.rand(100), (1,0): np.random.rand(100)}, rws=2.0, radial_grid=radial_grid)

potentials = [potential1, potential2]

V, grid = map_potential_to_grid(atoms, potentials)

# Now V contains the mapped potential on the Cartesian grid (in Angstroms)
# and grid contains the Cartesian coordinates of each point (in fractional coordinates)
