import numpy as np
from scipy.interpolate import interp1d
from scipy.special import sph_harm
from ase import Atoms

class KKRPotential:
    def __init__(self):
        self.data = {}
        self.rws = {}

def map_potential_to_grid(atoms: Atoms, potential: KKRPotential, grid_density: float = 1.0):
    # Define the Cartesian grid
    cell = atoms.get_cell()
    grid_shape = np.ceil(np.linalg.norm(cell, axis=1) * grid_density).astype(int)
    x = np.linspace(0, 1, grid_shape[0], endpoint=False)
    y = np.linspace(0, 1, grid_shape[1], endpoint=False)
    z = np.linspace(0, 1, grid_shape[2], endpoint=False)
    grid = np.meshgrid(x, y, z, indexing='ij')
    
    # Initialize the potential array
    V = np.zeros(grid_shape)
    
    # Loop over all atoms in the unit cell
    for atom, symbol in enumerate(atoms.get_chemical_symbols()):
        atom_position = atoms.get_positions()[atom]
        rws = potential.rws[symbol]
        atom_data = potential.data[symbol]
        
        # Create interpolation functions for each angular momentum channel
        interp_funcs = {L: interp1d(np.linspace(0, rws, len(values)), values, kind='cubic', bounds_error=False, fill_value=0) 
                        for L, values in atom_data.items()}
        
        # Loop over grid points
        for i, j, k in np.ndindex(V.shape):
            grid_point = np.dot(cell.T, [x[i], y[j], z[k]])
            r_vector = grid_point - atom_position
            r = np.linalg.norm(r_vector)
            
            # Check if the point is within the Wigner-Seitz radius
            if r <= rws:
                # Calculate spherical coordinates
                theta = np.arccos(r_vector[2] / r)
                phi = np.arctan2(r_vector[1], r_vector[0])
                
                # Sum contributions from all angular momentum channels
                for L, interp_func in interp_funcs.items():
                    l, m = L // 100, L % 100  # Assuming L is encoded as l*100 + m
                    V[i, j, k] += interp_func(r) * sph_harm(m, l, phi, theta).real
    
    return V, grid

# Example usage
atoms = Atoms('Si2', positions=[[0, 0, 0], [0.25, 0.25, 0.25]], cell=[[5.43, 0, 0], [0, 5.43, 0], [0, 0, 5.43]])
potential = KKRPotential()
# Here you would populate the potential.data and potential.rws dictionaries with your actual data

V, grid = map_potential_to_grid(atoms, potential)

# Now V contains the mapped potential on the Cartesian grid
# and grid contains the Cartesian coordinates of each point
