import numpy as np
import pyvista as pv
from ase import Atoms
from scipy import stats

def visualize_potential_3d_pyvista(atoms: Atoms, V: np.ndarray, grid: tuple, potential_range: tuple = None):
    """
    Create an interactive 3D visualization of the discretized potential within the unit cell using PyVista.
    
    :param atoms: ASE Atoms object representing the unit cell
    :param V: 3D numpy array of potential values
    :param grid: Tuple of 3D meshgrid arrays (X, Y, Z) in fractional coordinates
    :param potential_range: Tuple of (vmin, vmax) for potential color scaling (optional)
    """
    cell = atoms.get_cell()
    X, Y, Z = grid
    
    # Convert fractional coordinates to Cartesian
    X_cart = X * cell[0, 0] + Y * cell[1, 0] + Z * cell[2, 0]
    Y_cart = X * cell[0, 1] + Y * cell[1, 1] + Z * cell[2, 1]
    Z_cart = X * cell[0, 2] + Y * cell[1, 2] + Z * cell[2, 2]
    
    # Normalize V using z-score normalization
    V_norm = stats.zscore(V)
    
    # Print statistics about the normalized data
    print(f"Normalized V - Mean: {V_norm.mean():.4f}, Std: {V_norm.std():.4f}")
    print(f"Normalized V - Min: {V_norm.min():.4f}, Max: {V_norm.max():.4f}")
    
    # Create PyVista uniform grid
    grid = pv.UniformGrid()
    grid.dimensions = np.array(V.shape) + 1
    grid.origin = (X_cart.min(), Y_cart.min(), Z_cart.min())
    grid.spacing = (
        (X_cart.max() - X_cart.min()) / (V.shape[0] - 1),
        (Y_cart.max() - Y_cart.min()) / (V.shape[1] - 1),
        (Z_cart.max() - Z_cart.min()) / (V.shape[2] - 1)
    )
    grid.point_data["values"] = V_norm.flatten(order="F")
    
    # Create plotter
    plotter = pv.Plotter()
    
    # Add volume rendering
    plotter.add_volume(grid, cmap="RdBu_r", clim=[-2, 2], opacity="linear")
    
    # Add isosurfaces
    for iso_value in [-1, 0, 1]:
        try:
            isosurface = grid.contour(isosurfaces=[iso_value])
            if isosurface.n_points > 0:
                plotter.add_mesh(isosurface, color="black", opacity=0.1)
        except:
            print(f"Could not generate isosurface for value {iso_value}")
    
    # Add unit cell outline
    for i in range(3):
        for j in range(3):
            if j != i:
                for k in [0, 1]:
                    start = np.zeros(3)
                    end = np.zeros(3)
                    start[i] = k
                    end[i] = k
                    end[j] = 1
                    start_cart = np.dot(start, cell)
                    end_cart = np.dot(end, cell)
                    line = pv.Line(start_cart, end_cart)
                    plotter.add_mesh(line, color="black", line_width=5)
    
    # Set up the camera
    plotter.camera_position = [
        (20, 20, 20),  # Camera position
        (0, 0, 0),     # Focal point
        (0, 0, 1)      # View up direction
    ]
    
    # Show the plot
    plotter.show()

# Example usage:
# atoms = Atoms('Cu', positions=[[0, 0, 0]], cell=[[1.8, 1.8, 0], [1.8, 0, 1.8], [0, 1.8, 1.8]])
# V, grid = map_potential_to_grid(atoms, potentials)
# visualize_potential_3d_pyvista(atoms, V, grid)
