import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from ase import Atoms

def visualize_potential_3d(atoms: Atoms, V: np.ndarray, grid: tuple, potential_range: tuple = None):
    """
    Visualize the discretized potential in a 3D plot within the unit cell.
    
    :param atoms: ASE Atoms object representing the unit cell
    :param V: 3D numpy array of potential values
    :param grid: Tuple of 3D meshgrid arrays (X, Y, Z) in fractional coordinates
    :param potential_range: Tuple of (vmin, vmax) for potential color scaling (optional)
    """
    fig = plt.figure(figsize=(10, 10))
    ax = fig.add_subplot(111, projection='3d')
    
    # Extract cell vectors
    cell = atoms.get_cell()
    
    # Convert fractional coordinates to Cartesian
    X, Y, Z = grid
    points = np.dot(np.column_stack((X.flatten(), Y.flatten(), Z.flatten())), cell)
    
    # Plot potential values
    scatter = ax.scatter(points[:, 0], points[:, 1], points[:, 2], c=V.flatten(), 
                         cmap='viridis', alpha=0.5, s=5, vmin=potential_range[0], vmax=potential_range[1])
    
    # Add colorbar
    plt.colorbar(scatter, label='Potential')
    
    # Plot unit cell outline
    for i in range(3):
        for j in range(3):
            if j != i:
                for k in [0, 1]:
                    start = np.zeros(3)
                    end = np.zeros(3)
                    start[i] = k
                    end[i] = k
                    end[j] = 1
                    start_cart = np.dot(start, cell)
                    end_cart = np.dot(end, cell)
                    ax.plot([start_cart[0], end_cart[0]], 
                            [start_cart[1], end_cart[1]], 
                            [start_cart[2], end_cart[2]], 'k-')
    
    # Set labels and title
    ax.set_xlabel('X (Å)')
    ax.set_ylabel('Y (Å)')
    ax.set_zlabel('Z (Å)')
    ax.set_title('3D Visualization of Potential in Unit Cell')
    
    # Set aspect ratio to equal
    ax.set_box_aspect((np.ptp(points[:, 0]), np.ptp(points[:, 1]), np.ptp(points[:, 2])))
    
    plt.tight_layout()
    plt.show()

# Example usage (assuming you have already calculated V and grid)
atoms = Atoms('Cu', positions=[[0, 0, 0]], cell=[[1.8, 1.8, 0], [1.8, 0, 1.8], [0, 1.8, 1.8]])
# V and grid should be the result of your map_potential_to_grid function
# potential_range = (V.min(), V.max())  # or set custom range

# visualize_potential_3d(atoms, V, grid, potential_range)
