import numpy as np
from ase import Atoms

def write_chgcar(atoms: Atoms, charge_density: np.ndarray, filename: str = 'CHGCAR'):
    def numpy_to_chgcar_format(arr):
        flattened_data = arr.flatten(order='F')
        formatted_data = []
        for i, value in enumerate(flattened_data):
            if i > 0 and i % 5 == 0:
                formatted_data.append('\n')
            formatted_data.append(f'{value:17.10E}')
        return ''.join(formatted_data)

    with open(filename, 'w') as f:
        # Write system description (usually blank in VASP)
        f.write(atoms.get_chemical_formula() + '\n')
        
        # Write scaling factor (usually 1.0)
        f.write('1.0\n')
        
        # Write lattice vectors
        for vec in atoms.get_cell():
            f.write(f'{vec[0]:22.16f}{vec[1]:22.16f}{vec[2]:22.16f}\n')
        
        # Write atomic symbols
        symbols = atoms.get_chemical_symbols()
        unique_symbols = sorted(set(symbols))
        f.write('  '.join(unique_symbols) + '\n')
        
        # Write atom counts
        symbol_counts = [symbols.count(sym) for sym in unique_symbols]
        f.write('  '.join(str(count) for count in symbol_counts) + '\n')
        
        # Write "Direct" to indicate fractional coordinates
        f.write('Direct\n')
        
        # Write atomic positions (sorted by element)
        scaled_positions = atoms.get_scaled_positions()
        for sym in unique_symbols:
            for i, s in enumerate(symbols):
                if s == sym:
                    pos = scaled_positions[i]
                    f.write(f'{pos[0]:20.16f}{pos[1]:20.16f}{pos[2]:20.16f}\n')
        
        # Write an empty line
        f.write('\n')
        
        # Write grid dimensions
        ngx, ngy, ngz = charge_density.shape
        f.write(f'{ngx} {ngy} {ngz}\n')
        
        # Write charge density data
        f.write(numpy_to_chgcar_format(charge_density))

# Example usage:
# atoms = ... # Your ASE Atoms object
# charge_density = ... # Your charge density numpy array
# write_chgcar(atoms, charge_density, 'CHGCAR')
