import numpy as np
import matplotlib.pyplot as plt
from pymatgen.io.vasp import Chgcar
from ipywidgets import interact, FloatSlider
import ipywidgets as widgets
from IPython.display import display
from matplotlib.colors import TwoSlopeNorm

def plot_interactive_potential(chgcar, plane='xy', cmap='RdBu_r', lattice_constant=None):
    data = chgcar.data['total']
    structure = chgcar.structure

    if plane == 'xy':
        slice_axis, x_label, y_label = 2, 'x', 'y'
    elif plane == 'yz':
        slice_axis, x_label, y_label = 0, 'y', 'z'
    elif plane == 'xz':
        slice_axis, x_label, y_label = 1, 'x', 'z'
    else:
        raise ValueError("plane must be 'xy', 'yz', or 'xz'")

    fig, ax = plt.subplots(figsize=(12, 9))
    plt.close(fig)  # Prevent the figure from being displayed immediately

    # Calculate global min and max for colorbar
    vmin, vmax = data.min(), data.max()
    abs_max = max(abs(vmin), abs(vmax))

    # Create a normalization that's centered on zero
    norm = TwoSlopeNorm(vmin=-abs_max, vcenter=0, vmax=abs_max)

    # Create a static colorbar
    sm = plt.cm.ScalarMappable(cmap=cmap, norm=norm)
    sm.set_array([])
    cbar = fig.colorbar(sm, ax=ax)
    cbar.set_label('Electron Potential (eV)', rotation=270, labelpad=20)

    def update_plot(slice_pos):
        ax.clear()
        index = int(slice_pos * (data.shape[slice_axis] - 1))
        slice_data = np.take(data, index, axis=slice_axis)

        if lattice_constant:
            extent = [0, lattice_constant, 0, lattice_constant]
            im = ax.imshow(slice_data.T, cmap=cmap, norm=norm, origin='lower', aspect='equal', 
                           interpolation='nearest', extent=extent)
            ax.set_xlabel(f'Distance along {x_label} (Å)')
            ax.set_ylabel(f'Distance along {y_label} (Å)')
            
            tick_locations = np.linspace(0, lattice_constant, 6)
            ax.set_xticks(tick_locations)
            ax.set_yticks(tick_locations)
            ax.set_xticklabels([f'{x:.2f}' for x in tick_locations])
            ax.set_yticklabels([f'{y:.2f}' for y in tick_locations])

            slice_pos_angstrom = slice_pos * lattice_constant
            ax.set_title(f'ML-DFT Electron Potential (eV) in {plane}-plane\nSlice at {slice_pos:.2f}$a$ = {slice_pos_angstrom:.2f} Å')
        else:
            im = ax.imshow(slice_data.T, cmap=cmap, norm=norm, origin='lower', aspect='equal', 
                           interpolation='nearest')
            ax.set_xlabel(f'Grid points along {x_label}')
            ax.set_ylabel(f'Grid points along {y_label}')
            ax.set_xticks(np.linspace(0, slice_data.shape[0], 6))
            ax.set_yticks(np.linspace(0, slice_data.shape[1], 6))
            ax.set_title(f'ML-DFT Electron Potential (eV) in {plane}-plane\nSlice at {slice_pos:.2f}')

        fig.canvas.draw_idle()
        display(fig, clear=True)

    slider = FloatSlider(min=0, max=1, step=0.01, value=0.5, description=f'{plane[-1]}-position:')
    interact(update_plot, slice_pos=slider)

# Usage example:
# chgcar = Chgcar.from_file("CHGCAR")
# plot_interactive_potential(chgcar, plane='xy', lattice_constant=5.43)
