import numpy as np
import matplotlib.pyplot as plt
from pymatgen.io.vasp import Chgcar

def plot_periodic_contour(chgcar, plane='xy', slice_position=0.5, repeat=(2, 2), cmap='RdBu_r'):
    # Extract data and structure
    data = chgcar.data['total']
    structure = chgcar.structure

    # Determine the slice
    if plane == 'xy':
        index = int(slice_position * data.shape[2])
        slice_data = data[:, :, index]
        x_label, y_label = 'x', 'y'
    elif plane == 'yz':
        index = int(slice_position * data.shape[0])
        slice_data = data[index, :, :]
        x_label, y_label = 'y', 'z'
    elif plane == 'xz':
        index = int(slice_position * data.shape[1])
        slice_data = data[:, index, :]
        x_label, y_label = 'x', 'z'
    else:
        raise ValueError("plane must be 'xy', 'yz', or 'xz'")

    # Create periodic repetition
    repeated_data = np.tile(slice_data, repeat)

    # Create coordinate grids
    nx, ny = repeated_data.shape
    x = np.linspace(0, repeat[0], nx)
    y = np.linspace(0, repeat[1], ny)
    X, Y = np.meshgrid(x, y)

    # Create the plot
    fig, ax = plt.subplots(figsize=(10, 8))
    
    # Plot contour
    contour = ax.contourf(X, Y, repeated_data.T, cmap=cmap, levels=20)
    
    # Add colorbar
    cbar = fig.colorbar(contour)
    cbar.set_label('Electron Potential (eV)', rotation=270, labelpad=15)

    # Set labels and title
    ax.set_xlabel(f'Grid points along {x_label}')
    ax.set_ylabel(f'Grid points along {y_label}')
    ax.set_title(f'ML-DFT Electron Potential (eV) in {plane}-plane')

    # Show grid
    ax.grid(color='black', linestyle='--', linewidth=0.5, alpha=0.3)

    plt.show()

# Example usage:
# chgcar = Chgcar.from_file("CHGCAR")
# plot_periodic_contour(chgcar, plane='xy', slice_position=0.5, repeat=(4, 4))
