import numpy as np
import matplotlib.pyplot as plt
from pymatgen.io.vasp import Chgcar

def plot_multi_slice_potential(chgcar, plane='xy', slice_positions=[0.2, 0.35, 0.5, 0.65], cmap='RdBu_r', lattice_constant=None):
    data = chgcar.data['total']
    structure = chgcar.structure

    if plane == 'xy':
        slice_axis, x_label, y_label = 2, 'x', 'y'
    elif plane == 'yz':
        slice_axis, x_label, y_label = 0, 'y', 'z'
    elif plane == 'xz':
        slice_axis, x_label, y_label = 1, 'x', 'z'
    else:
        raise ValueError("plane must be 'xy', 'yz', or 'xz'")

    fig, axs = plt.subplots(2, 2, figsize=(16, 14))
    fig.suptitle(f'ML-DFT Electron Potential (eV) in {plane}-plane at Different Slice Positions', fontsize=16)

    for ax, slice_pos in zip(axs.flatten(), slice_positions):
        index = int(slice_pos * data.shape[slice_axis])
        slice_data = np.take(data, index, axis=slice_axis)

        im = ax.imshow(slice_data.T, cmap=cmap, origin='lower', aspect='equal', interpolation='nearest')
        
        if lattice_constant:
            extent = [0, lattice_constant, 0, lattice_constant]
            ax.set_xlabel(f'Distance along {x_label} (Å)')
            ax.set_ylabel(f'Distance along {y_label} (Å)')
            ax.imshow(slice_data.T, cmap=cmap, origin='lower', aspect='equal', interpolation='nearest', extent=extent)
            
            tick_locations = np.linspace(0, lattice_constant, 6)
            ax.set_xticks(tick_locations)
            ax.set_yticks(tick_locations)
            ax.set_xticklabels([f'{x:.2f}' for x in tick_locations])
            ax.set_yticklabels([f'{y:.2f}' for y in tick_locations])

            slice_pos_angstrom = slice_pos * lattice_constant
            ax.set_title(f'Slice at {slice_pos:.2f}$a$ = {slice_pos_angstrom:.2f} Å')
        else:
            ax.set_xlabel(f'Grid points along {x_label}')
            ax.set_ylabel(f'Grid points along {y_label}')
            ax.set_xticks(np.linspace(0, slice_data.shape[0], 6))
            ax.set_yticks(np.linspace(0, slice_data.shape[1], 6))
            ax.set_title(f'Slice at {slice_pos:.2f}')

    cbar = fig.colorbar(im, ax=axs.ravel().tolist())
    cbar.set_label('Electron Potential (eV)', rotation=270, labelpad=20)

    plt.tight_layout()
    return fig, axs

# Usage example:
# chgcar = Chgcar.from_file("CHGCAR")
# fig, axs = plot_multi_slice_potential(chgcar, plane='xy', slice_positions=[0.2, 0.35, 0.5, 0.7], lattice_constant=5.43)
# fig.savefig('potential_plot.png', dpi=300, bbox_inches='tight')
# fig.savefig('potential_plot.pdf', bbox_inches='tight')
# plt.show()  # Optional, if you want to display the plot
