import numpy as np
import matplotlib.pyplot as plt
from pymatgen.io.vasp import Chgcar

def plot_multi_slice_potential(chgcar, plane='xy', slice_positions=[0.2, 0.35, 0.5, 0.65], cmap='RdBu_r'):
    # Extract data and structure
    data = chgcar.data['total']
    structure = chgcar.structure

    # Determine the slice axis
    if plane == 'xy':
        slice_axis = 2
        x_label, y_label = 'x', 'y'
    elif plane == 'yz':
        slice_axis = 0
        x_label, y_label = 'y', 'z'
    elif plane == 'xz':
        slice_axis = 1
        x_label, y_label = 'x', 'z'
    else:
        raise ValueError("plane must be 'xy', 'yz', or 'xz'")

    # Create the plot
    fig, axs = plt.subplots(2, 2, figsize=(16, 14))
    fig.suptitle(f'ML-DFT Electron Potential (eV) in {plane}-plane at Different Slice Positions', fontsize=16)

    for ax, slice_pos in zip(axs.flatten(), slice_positions):
        index = int(slice_pos * data.shape[slice_axis])
        if plane == 'xy':
            slice_data = data[:, :, index]
        elif plane == 'yz':
            slice_data = data[index, :, :]
        else:  # 'xz'
            slice_data = data[:, index, :]

        # Plot pixelated image
        im = ax.imshow(slice_data.T, cmap=cmap, origin='lower', aspect='equal', interpolation='nearest')
        
        # Set labels
        ax.set_xlabel(f'Grid points along {x_label}')
        ax.set_ylabel(f'Grid points along {y_label}')
        ax.set_title(f'Slice at {slice_pos:.2f}')

        # Set tick labels to show actual grid points
        ax.set_xticks(np.arange(0, slice_data.shape[0], slice_data.shape[0]//5))
        ax.set_yticks(np.arange(0, slice_data.shape[1], slice_data.shape[1]//5))
        ax.set_xticklabels(np.arange(0, slice_data.shape[0], slice_data.shape[0]//5))
        ax.set_yticklabels(np.arange(0, slice_data.shape[1], slice_data.shape[1]//5))

    # Add colorbar
    cbar = fig.colorbar(im, ax=axs.ravel().tolist())
    cbar.set_label('Electron Potential (eV)', rotation=270, labelpad=20)

    plt.tight_layout()
    plt.show()

# Example usage:
# chgcar = Chgcar.from_file("CHGCAR")
# plot_multi_slice_potential(chgcar, plane='xy', slice_positions=[0.2, 0.35, 0.5, 0.65])
