import numpy as np
import matplotlib.pyplot as plt
from pymatgen.io.vasp import Chgcar

def plot_periodic_pixelated(chgcar, plane='xy', slice_position=0.5, repeat=(2, 2), cmap='RdBu_r'):
    # Extract data and structure
    data = chgcar.data['total']
    structure = chgcar.structure

    # Determine the slice
    if plane == 'xy':
        index = int(slice_position * data.shape[2])
        slice_data = data[:, :, index]
        x_label, y_label = 'x', 'y'
    elif plane == 'yz':
        index = int(slice_position * data.shape[0])
        slice_data = data[index, :, :]
        x_label, y_label = 'y', 'z'
    elif plane == 'xz':
        index = int(slice_position * data.shape[1])
        slice_data = data[:, index, :]
        x_label, y_label = 'x', 'z'
    else:
        raise ValueError("plane must be 'xy', 'yz', or 'xz'")

    # Create periodic repetition
    repeated_data = np.tile(slice_data, repeat)

    # Create the plot
    fig, ax = plt.subplots(figsize=(12, 10))
    
    # Plot pixelated image
    im = ax.imshow(repeated_data.T, cmap=cmap, origin='lower', aspect='equal', interpolation='nearest')
    
    # Add colorbar
    cbar = fig.colorbar(im)
    cbar.set_label('Electron Potential (eV)', rotation=270, labelpad=15)

    # Set labels and title
    ax.set_xlabel(f'Grid points along {x_label}')
    ax.set_ylabel(f'Grid points along {y_label}')
    ax.set_title(f'ML-DFT Electron Potential (eV) in {plane}-plane')

    # Set tick labels to show actual grid points
    ax.set_xticks(np.arange(0, repeated_data.shape[0], slice_data.shape[0]))
    ax.set_yticks(np.arange(0, repeated_data.shape[1], slice_data.shape[1]))
    ax.set_xticklabels(np.arange(0, repeated_data.shape[0], slice_data.shape[0]))
    ax.set_yticklabels(np.arange(0, repeated_data.shape[1], slice_data.shape[1]))

    # Add grid lines to separate unit cells
    for i in range(1, repeat[0]):
        ax.axvline(x=i * slice_data.shape[0] - 0.5, color='black', linestyle='--', linewidth=0.5)
    for i in range(1, repeat[1]):
        ax.axhline(y=i * slice_data.shape[1] - 0.5, color='black', linestyle='--', linewidth=0.5)

    plt.show()

# Example usage:
# chgcar = Chgcar.from_file("CHGCAR")
# plot_periodic_pixelated(chgcar, plane='xy', slice_position=0.5, repeat=(4, 4))
