import numpy as np
import pyvista as pv
from pymatgen.io.vasp import Chgcar

def visualize_chgcar(chgcar, periodic_copies=(1, 1, 1), isosurface_value=None):
    # Extract data and structure from Chgcar object
    data = chgcar.data['total']
    structure = chgcar.structure
    
    # Print debugging information
    print(f"Data shape: {data.shape}")
    
    # Create a 3D grid
    grid = pv.ImageData()
    grid.dimensions = np.array(data.shape) + 1  # Add 1 to convert cell-centered to node-centered
    grid.origin = (0, 0, 0)
    grid.spacing = structure.lattice.abc / np.array(data.shape)  # Use actual lattice parameters
    
    # Print grid information
    print(f"Grid dimensions: {grid.dimensions}")
    print(f"Number of points in grid: {grid.n_points}")
    print(f"Size of data array: {data.size}")
    
    # Interpolate cell-centered data to node-centered data
    node_data = np.zeros(grid.dimensions)
    node_data[:-1, :-1, :-1] = data
    node_data[-1, :, :] = node_data[0, :, :]  # Periodic boundary conditions
    node_data[:, -1, :] = node_data[:, 0, :]
    node_data[:, :, -1] = node_data[:, :, 0]
    
    # Add the scalar data to the grid
    grid.point_data["values"] = node_data.flatten(order="F")
    
    # Create periodic copies
    periodic_grid = grid.copy()
    for i in range(periodic_copies[0]):
        for j in range(periodic_copies[1]):
            for k in range(periodic_copies[2]):
                if i == 0 and j == 0 and k == 0:
                    continue
                copy = grid.copy()
                copy.translate(np.array([i, j, k]) * np.array(structure.lattice.abc))
                periodic_grid = periodic_grid.merge(copy)
    
    # Create a plotter
    plotter = pv.Plotter()
    
    # Add the periodic grid to the plotter
    plotter.add_mesh(periodic_grid, opacity=0.5, cmap="viridis")
    
    # Add isosurface if specified
    if isosurface_value is not None:
        isosurface = periodic_grid.contour(isosurfaces=[isosurface_value])
        plotter.add_mesh(isosurface, color="red", opacity=0.5)
    
    # Add atom positions
    for site in structure.sites:
        sphere = pv.Sphere(radius=0.1, center=site.coords)
        plotter.add_mesh(sphere, color="white")
    
    # Show the plot
    plotter.show()

# Example usage:
# chgcar = Chgcar.from_file("CHGCAR")
# visualize_chgcar(chgcar, periodic_copies=(2, 2, 2), isosurface_value=0.1)
