import numpy as np
import pyvista as pv
from pymatgen.io.vasp import Chgcar

def visualize_chgcar(chgcar, periodic_copies=(1, 1, 1), isosurface_value=None):
    # Extract data and structure from Chgcar object
    data = chgcar.data['total']
    structure = chgcar.structure
    
    # Print debugging information
    print(f"Data shape: {data.shape}")
    print(f"Data range: {np.min(data)} to {np.max(data)}")
    
    # Create a 3D grid
    grid = pv.ImageData()
    grid.dimensions = np.array(data.shape) + 1  # Add 1 to convert cell-centered to node-centered
    grid.origin = (0, 0, 0)
    grid.spacing = structure.lattice.abc / np.array(data.shape)  # Use actual lattice parameters
    
    # Print grid information
    print(f"Grid dimensions: {grid.dimensions}")
    print(f"Number of points in grid: {grid.n_points}")
    
    # Interpolate cell-centered data to node-centered data
    node_data = np.zeros(grid.dimensions)
    node_data[:-1, :-1, :-1] = data
    node_data[-1, :, :] = node_data[0, :, :]  # Periodic boundary conditions
    node_data[:, -1, :] = node_data[:, 0, :]
    node_data[:, :, -1] = node_data[:, :, 0]
    
    # Add the scalar data to the grid
    grid.point_data["values"] = node_data.flatten(order="F")
    
    # Create periodic copies
    periodic_grid = grid.copy()
    for i in range(periodic_copies[0]):
        for j in range(periodic_copies[1]):
            for k in range(periodic_copies[2]):
                if i == 0 and j == 0 and k == 0:
                    continue
                copy = grid.copy()
                copy.translate(np.array([i, j, k]) * np.array(structure.lattice.abc))
                periodic_grid = periodic_grid.merge(copy)
    
    # Create a plotter
    plotter = pv.Plotter()
    
    # Add the periodic grid to the plotter
    plotter.add_mesh(periodic_grid, opacity=0.5, cmap="viridis")
    
    # Add isosurface if specified and within data range
    if isosurface_value is not None:
        if np.min(data) <= isosurface_value <= np.max(data):
            try:
                isosurface = periodic_grid.contour(isosurfaces=[isosurface_value])
                if isosurface.n_points > 0:
                    plotter.add_mesh(isosurface, color="red", opacity=0.5)
                else:
                    print(f"Warning: Isosurface at value {isosurface_value} resulted in an empty mesh.")
            except Exception as e:
                print(f"Error generating isosurface: {e}")
        else:
            print(f"Warning: Isosurface value {isosurface_value} is outside the data range.")
    
    # Add atom positions
    for site in structure.sites:
        sphere = pv.Sphere(radius=0.1, center=site.coords)
        plotter.add_mesh(sphere, color="white")
    
    # Show the plot
    plotter.show()

# Example usage:
# chgcar = Chgcar.from_file("CHGCAR")
# visualize_chgcar(chgcar, periodic_copies=(2, 2, 2), isosurface_value=0.1)
