import numpy as np
import pyvista as pv
from pymatgen.io.vasp import Chgcar
from pymatgen.core import Structure

def visualize_chgcar(chgcar, periodic_copies=(1, 1, 1), isosurface_value=None):
    # Extract data and structure from Chgcar object
    data = chgcar.data['total']
    structure = chgcar.structure
    
    # Create a 3D grid
    grid = pv.UniformGrid()
    grid.dimensions = np.array(data.shape) + 1
    grid.origin = (0, 0, 0)
    grid.spacing = (1/data.shape[0], 1/data.shape[1], 1/data.shape[2])
    
    # Add the scalar data to the grid
    grid.point_data["values"] = data.flatten(order="F")
    
    # Create periodic copies
    periodic_grid = grid.copy()
    for i in range(periodic_copies[0]):
        for j in range(periodic_copies[1]):
            for k in range(periodic_copies[2]):
                if i == 0 and j == 0 and k == 0:
                    continue
                copy = grid.copy()
                copy.translate((i, j, k))
                periodic_grid = periodic_grid.merge(copy)
    
    # Create a plotter
    plotter = pv.Plotter()
    
    # Add the periodic grid to the plotter
    plotter.add_mesh(periodic_grid, opacity=0.5, cmap="viridis")
    
    # Add isosurface if specified
    if isosurface_value is not None:
        isosurface = periodic_grid.contour(isosurfaces=[isosurface_value])
        plotter.add_mesh(isosurface, color="red", opacity=0.5)
    
    # Add atom positions
    for site in structure.sites:
        sphere = pv.Sphere(radius=0.1, center=site.coords)
        plotter.add_mesh(sphere, color="white")
    
    # Show the plot
    plotter.show()

# Example usage:
# chgcar = Chgcar.from_file("CHGCAR")
# visualize_chgcar(chgcar, periodic_copies=(2, 2, 2), isosurface_value=0.1)
