import numpy as np
import matplotlib.pyplot as plt
from pymatgen.io.vasp import Chgcar
from matplotlib.colors import TwoSlopeNorm

def plot_multi_slice_potential(chgcar, plane='xy', slice_positions=[0.2, 0.35, 0.5, 0.65], cmap='RdBu_r', 
                               lattice_constant=None, plot_style='imshow', contour_levels=20, repeat=(1, 1)):
    data = chgcar.data['total']
    structure = chgcar.structure

    if plane == 'xy':
        slice_axis, x_label, y_label = 2, 'x', 'y'
    elif plane == 'yz':
        slice_axis, x_label, y_label = 0, 'y', 'z'
    elif plane == 'xz':
        slice_axis, x_label, y_label = 1, 'x', 'z'
    else:
        raise ValueError("plane must be 'xy', 'yz', or 'xz'")

    vmin, vmax = data.min(), data.max()
    abs_max = max(abs(vmin), abs(vmax))
    norm = TwoSlopeNorm(vmin=-abs_max, vcenter=0, vmax=abs_max)

    if slice_positions and len(slice_positions) > 0:
        # Multiple subplot behavior
        fig, axs = plt.subplots(2, 2, figsize=(16, 14))
        fig.suptitle(f'ML-DFT Electron Potential (eV) in {plane}-plane at Different Slice Positions', fontsize=16)
        axes = axs.flatten()
    else:
        # Single plot with repeat
        fig, ax = plt.subplots(figsize=(12, 10))
        fig.suptitle(f'ML-DFT Electron Potential (eV) in {plane}-plane', fontsize=16)
        axes = [ax]
        slice_positions = [0.5]  # Default to middle slice for single plot

    for ax, slice_pos in zip(axes, slice_positions):
        index = int(slice_pos * (data.shape[slice_axis] - 1))
        slice_data = np.take(data, index, axis=slice_axis)
        
        if len(slice_positions) == 1:  # Single plot with repeat
            repeated_data = np.tile(slice_data, repeat)
        else:
            repeated_data = slice_data

        if lattice_constant:
            extent = [0, lattice_constant * repeat[0], 0, lattice_constant * repeat[1]]
            X, Y = np.meshgrid(np.linspace(0, lattice_constant * repeat[0], repeated_data.shape[0]),
                               np.linspace(0, lattice_constant * repeat[1], repeated_data.shape[1]))
            
            if plot_style == 'imshow':
                im = ax.imshow(repeated_data.T, cmap=cmap, norm=norm, origin='lower', aspect='equal', 
                               interpolation='nearest', extent=extent)
            elif plot_style == 'contourf':
                im = ax.contourf(X, Y, repeated_data.T, levels=contour_levels, cmap=cmap, norm=norm)
            
            ax.set_xlabel(f'Distance along {x_label} (Å)')
            ax.set_ylabel(f'Distance along {y_label} (Å)')
            
            tick_locations = np.linspace(0, lattice_constant * repeat[0], 6)
            ax.set_xticks(tick_locations)
            ax.set_yticks(tick_locations)
            ax.set_xticklabels([f'{x:.2f}' for x in tick_locations])
            ax.set_yticklabels([f'{y:.2f}' for y in tick_locations])

            if len(slice_positions) == 1:
                for i in range(1, repeat[0]):
                    ax.axvline(x=i * lattice_constant, color='black', linestyle='--', linewidth=0.5)
                for i in range(1, repeat[1]):
                    ax.axhline(y=i * lattice_constant, color='black', linestyle='--', linewidth=0.5)

            slice_pos_angstrom = slice_pos * lattice_constant
            ax.set_title(f'Slice at {slice_pos:.2f}$a$ = {slice_pos_angstrom:.2f} Å')
        else:
            X, Y = np.meshgrid(range(repeated_data.shape[0]), range(repeated_data.shape[1]))
            
            if plot_style == 'imshow':
                im = ax.imshow(repeated_data.T, cmap=cmap, norm=norm, origin='lower', aspect='equal', 
                               interpolation='nearest')
            elif plot_style == 'contourf':
                im = ax.contourf(X, Y, repeated_data.T, levels=contour_levels, cmap=cmap, norm=norm)
            
            ax.set_xlabel(f'Grid points along {x_label}')
            ax.set_ylabel(f'Grid points along {y_label}')
            ax.set_xticks(np.linspace(0, repeated_data.shape[0], 6))
            ax.set_yticks(np.linspace(0, repeated_data.shape[1], 6))

            if len(slice_positions) == 1:
                for i in range(1, repeat[0]):
                    ax.axvline(x=i * slice_data.shape[0], color='black', linestyle='--', linewidth=0.5)
                for i in range(1, repeat[1]):
                    ax.axhline(y=i * slice_data.shape[1], color='black', linestyle='--', linewidth=0.5)

            ax.set_title(f'Slice at {slice_pos:.2f}')

    if len(slice_positions) == 1:
        # For single plot, adjust layout and add colorbar
        fig.tight_layout()
        cbar = fig.colorbar(im, ax=ax, fraction=0.046, pad=0.04)
    else:
        # For multiple subplots, adjust layout and add colorbar
        fig.tight_layout()
        fig.subplots_adjust(right=0.85, hspace=0.3, wspace=0.3)
        cbar_ax = fig.add_axes([0.88, 0.15, 0.03, 0.7])
        cbar = fig.colorbar(im, cax=cbar_ax)
    
    cbar.set_label('Electron Potential (eV)', rotation=270, labelpad=20)
    
    return fig, axes

# Usage examples:
# chgcar = Chgcar.from_file("CHGCAR")

# Multiple slices (original behavior)
fig, axs = plot_multi_slice_potential(chgcar=V_ch, plane='xy', slice_positions=[0.1, 0.45, 0.5, 0.6], 
                                      lattice_constant=structure.lattice.a, plot_style='imshow')

# Single slice with repeat (new behavior)
fig, ax = plot_multi_slice_potential(chgcar=V_ch, plane='xy', slice_positions=None, 
                                     lattice_constant=structure.lattice.a, plot_style='imshow', repeat=(2,2))

# Save and show plots
fig.savefig('potential_plot.png', dpi=300, bbox_inches='tight')
fig.savefig('potential_plot.pdf', bbox_inches='tight')
plt.show()  # Optional, if you want to display the plot
