import git
from datetime import datetime

def git_commit_hash(length: int = 7) -> str:
    """
    Returns the current commit's hash with specified length.
    Must be run from within a git repository.

    Args:
        length (int): Number of characters to return from the beginning of the hash.
                     Defaults to 7.

    Returns:
        str: Short hash of the current commit

    Raises:
        ValueError: If length is less than 1 or greater than 40
        GitError: If not in a git repository
    """
    if not 1 <= length <= 40:
        raise ValueError("Hash length must be between 1 and 40")

    repo = git.Repo('.', search_parent_directories=True)
    commit = repo.head.commit
    return commit.hexsha[:length]

def git_commit_date(format: str = '%Y-%m-%d-%H-%M-%S') -> str:
    """
    Returns the current commit's date in the specified format.
    Must be run from within a git repository.

    Args:
        format (str): DateTime format string.
                     Defaults to 'YYYY-MM-DD-HH-MM-SS' format.

    Returns:
        str: Formatted date string of the current commit

    Raises:
        ValueError: If format string is invalid
        GitError: If not in a git repository
    """
    repo = git.Repo('.', search_parent_directories=True)
    commit = repo.head.commit
    commit_date = datetime.fromtimestamp(commit.committed_date)
    return commit_date.strftime(format)

# Example usage:
if __name__ == '__main__':
    try:
        # Get hash with default length (7)
        print(f"Default hash: {git_commit_hash()}")

        # Get hash with custom length
        print(f"Custom length hash: {git_commit_hash(length=8)}")

        # Get date with default format
        print(f"Default date format: {git_commit_date()}")

        # Get date with custom format
        print(f"Custom date format: {git_commit_date(format='%Y-%m-%d %H:%M:%S')}")

    except Exception as e:
        print(f"Error: {e}")
