from dataclasses import dataclass, field
from git_utils import git_commit_date, git_commit_hash
from pathlib import Path
from typing import Union

def process_input_path(path: Path) -> str:
    """Convert potential input filepath to short format string."""
    path_str = str(path)
    if path_str.endswith('host_scf/voronoi/output.pot'):
        return 'host-voro'
    elif path_str.endswith('host_scf/potential'):
        return 'host-in'
    elif path_str.endswith('host_scf/out_potential'):
        return 'host-out'
    raise ValueError(f"Invalid potential input path: {path}")

@dataclass
class PlotFilePath:
    """Stores components for plot filepath generation.
    
    Valid values:
    nb: 'ns', 'nc'
    inp: Path object ending in one of:
         - host_scf/voronoi/output.pot -> 'host-voro'
         - host_scf/potential -> 'host-in'
         - host_scf/out_potential -> 'host-out'
    pot: 'pi', 'pc', 'pd'
    trafo: 'cs', 'v2', 'v3', 'cwc'
    res: int (typically 60, 90, 120)
    typ: 
        - for ns: '2dcs', '3dvp'
        - for nc: '0cfzX', '0cizX', '0vp', '12dcs', '2svpm', '2ivsp', '2cpcp'
    """
    nb: str
    inp: Union[str, Path]
    pot: str
    trafo: str
    res: int
    typ: str
    date: str = field(default_factory=git_commit_date)
    hsh: str = field(default_factory=git_commit_hash)

    def __post_init__(self):
        """Convert inp path to standardized string format after initialization."""
        if isinstance(self.inp, Path):
            self.inp = process_input_path(self.inp)
        elif isinstance(self.inp, str):
            # If it's already a processed string, validate it
            if self.inp not in {'host-voro', 'host-in', 'host-out'}:
                raise ValueError(f"Invalid inp value: {self.inp}")

    def to_filename(self) -> str:
        """Generate filename with components joined by dashes."""
        return "-".join([
            self.nb,
            self.inp,
            self.pot,
            self.trafo,
            str(self.res),
            self.typ,
            self.date,
            self.hsh
        ])
    
    def __str__(self) -> str:
        return self.to_filename()
