from dataclasses import dataclass, field
from git_utils import git_commit_date, git_commit_hash
from pathlib import Path
from typing import Union

@dataclass
class PlotFilePath:
    """Stores components for plot filepath generation.

    Valid values:
    nb: 'ns', 'nc'
    inp: Path object ending in one of:
         - host_scf/voronoi/output.pot -> 'host-voro'
         - host_scf/potential -> 'host-in'
         - host_scf/out_potential -> 'host-out'
    pot: 'pi', 'pc', 'pd'
    trafo: 'cs', 'v2', 'v3', 'cwc'
    res: int (typically 60, 90, 120)
    typ:
        - for ns: '2dcs', '3dvp'
        - for nc: '0cfzX', '0cizX', '0vp', '12dcs', '2svpm', '2ivsp', '2cpcp'
    """
    nb: str
    inp: Union[str, Path]
    pot: str
    trafo: str
    res: int
    typ: str
    date: str = field(default_factory=git_commit_date)
    hsh: str = field(default_factory=git_commit_hash)

    def __post_init__(self):
        """Convert inp path to standardized string format after initialization."""
        if isinstance(self.inp, Path):
            match str(self.inp):
                case s if s.endswith('host_scf/voronoi/output.pot'):
                    self.inp = 'host-voro'
                case s if s.endswith('host_scf/potential'):
                    self.inp = 'host-in'
                case s if s.endswith('host_scf/out_potential'):
                    self.inp = 'host-out'
                case _:
                    raise ValueError(f"Invalid potential input path: {self.inp}")
        elif isinstance(self.inp, str):
            if self.inp not in {'host-voro', 'host-in', 'host-out'}:
                raise ValueError(f"Invalid inp value: {self.inp}")

    def to_filename(self) -> str:
        """Generate filename with components joined by dashes."""
        return "-".join([
            self.nb,
            self.inp,
            self.pot,
            self.trafo,
            str(self.res),
            self.typ,
            self.date,
            self.hsh
        ])

    def __str__(self) -> str:
        return self.to_filename()


# Example usage:
if __name__ == '__main__':
    filepath_potential = Path('/Users/wasmer/src/iffgit.fz-juelich.de/phd-project-wasmer/projects/vimp-prediction-old/tests/files/host_scf/out_potential')

    pfp = PlotFilePath(
        nb="ns",
        inp=filepath_potential,  # This will be converted to "host-out"
        pot="pc",
        trafo="cs",
        res=60,
        typ="2dcs"
    )

    print(pfp)
