from dataclasses import dataclass
from enum import Enum, auto
from typing import Union

# Enums for valid values
class Notebook(Enum):
    NS = "ns"  # Normal Space
    NC = "nc"  # Normal Coordinates

class Potential(Enum):
    PI = "pi"  # Potential I
    PC = "pc"  # Potential C
    PD = "pd"  # Potential D

class Trafo(Enum):
    CS = "cs"   # Standard
    V2 = "v2"   # Version 2
    V3 = "v3"   # Version 3
    CWC = "cwc"  # Custom

class PlotType(Enum):
    # NS plot types
    PLOT_2DCS = "2dcs"  # 2D Contour Surface
    PLOT_3DVP = "3dvp"  # 3D Vector Plot
    
    # NC plot types
    PLOT_0CFZX = "0cfzX"  # Zero Contour Force Z-X
    PLOT_0CIZX = "0cizX"  # Zero Contour Impact Z-X
    PLOT_0VP = "0vp"      # Zero Vector Plot
    PLOT_12DCS = "12dcs"  # 1-2D Contour Surface
    PLOT_2SVPM = "2svpm"  # 2D Surface Vector Plot M
    PLOT_2IVSP = "2ivsp"  # 2D Impact Vector Surface Plot
    PLOT_2CPCP = "2cpcp"  # 2D CP CP Plot

@dataclass
class PlotFilePath:
    notebook: Notebook
    potential: Potential
    trafo: Trafo
    trafo_res: int
    plot_type: PlotType
    commit_date: str
    commit_hash: str
    
    @classmethod
    def create(cls, 
               notebook: Union[str, Notebook],
               potential: Union[str, Potential],
               trafo: Union[str, Trafo],
               trafo_res: int,
               plot_type: Union[str, PlotType],
               commit_date: str = None,
               commit_hash: str = None) -> 'PlotFilePath':
        """
        Factory method to create PlotFilePath with string or Enum inputs.
        Automatically fetches git info if not provided.
        """
        # Convert strings to enums if needed
        nb = notebook if isinstance(notebook, Notebook) else Notebook(notebook)
        pot = potential if isinstance(potential, Potential) else Potential(potential)
        tf = trafo if isinstance(trafo, Trafo) else Trafo(trafo)
        pt = plot_type if isinstance(plot_type, PlotType) else PlotType(plot_type)
        
        # Get git info if not provided
        if commit_date is None:
            from git_utils import git_commit_date  # Import your git functions
            commit_date = git_commit_date()
        if commit_hash is None:
            from git_utils import git_commit_hash
            commit_hash = git_commit_hash()
            
        return cls(nb, pot, tf, trafo_res, pt, commit_date, commit_hash)
    
    def to_filename(self) -> str:
        """Generate filename with components joined by dashes."""
        components = [
            self.notebook.value,
            self.potential.value,
            self.trafo.value,
            str(self.trafo_res),
            self.plot_type.value,
            self.commit_date,
            self.commit_hash
        ]
        return "-".join(components)
    
    def __str__(self) -> str:
        """String representation is the filename."""
        return self.to_filename()

# Example usage:
if __name__ == "__main__":
    # Using strings (as in your original code)
    plot_path = PlotFilePath.create(
        notebook="ns",
        potential="pc",
        trafo="cs",
        trafo_res=60,
        plot_type="2dcs"
    )
    
    # Or using enums directly
    plot_path_enum = PlotFilePath.create(
        notebook=Notebook.NS,
        potential=Potential.PC,
        trafo=Trafo.CS,
        trafo_res=60,
        plot_type=PlotType.PLOT_2DCS
    )
    
    print(f"Generated filename: {plot_path}")
